#ifndef GAME_H

#include <vector>
#include <algorithm>
#include <queue>
#include <set>
#include <cmath>
#include "Define.h"
#include "Map.h"
#include "Unit.h"
#include "Building.h"
#include "Node.h"
#include "Graph.h"
#include "UnitProperty.h"
#include "TileProperty.h"
#include "Mymath.h"
#include "Quadtree.h"
#include "Rect.h"
#include "Player.h"
#include "Pathfinding.h"
#include "Projectile.h"

#define GAME_H

/**
 * Contem rotinas de tratamento e viabilidade de comandos
 */
class Game {
public:
    Game(Map* map);
    virtual ~Game();
    std::vector<Unit*> getUnitSet();
    void setUnitSet(std::vector<Unit*> unitSet);
    std::vector<Building*> getBuildingSet();
    std::vector<Player*> getPlayer();
    std::vector<Projectile*> getProjectileSet();
    void setProjectileSet(std::vector<Projectile*> projectileSet);
    void setBuildingSet(std::vector<Building*> buildingSet);
    void insertUnit(Unit* unit);
    bool removeUnit(Unit* unit);
    bool isInsertBuildingValid(float px, float py, int buildType);
    void insertBuilding(Building* building);
    bool removeBuilding(Building* building);
    bool setBuildingWalkable(Building* building, bool boolean);
    Graph* getGraph();
    void setGraph(Graph* Graph);
    Map* getMap();
    void setMap(Map* map);
    bool unitMove(Unit* unit, float px, float py);

    bool selectStop(std::vector<Unit*> select);
    bool selectMove(std::vector<Unit*> select, float px, float py);
    bool selectMoveAttack(std::vector<Unit*> select, float px, float py);
    bool selectMoveBuild(std::vector<Unit*> select, float px, float py, int buildType);
    bool selectMoveGather(std::vector<Unit*> select, float px, float py);
    bool buildingPath(Building* building, float px, float py);

    Unit* getEnemyUnit(Unit* unit);
    Building* getEnemyBuilding(Unit* unit);
    bool hasUnitInBuilding(int x, int y, int buildType);
    bool insertBuildingQueue(Building* building, int unitType);
    bool cancelBuildingQueue(Building* building);
    bool insertUnitBuild(Unit* unit, int buildType);
    bool removeUnitBuild(Unit* unit);
    void multiAction(std::vector<Unit*> select, float px, float py);
    Entity* getEntity(float px, float py);
    void setBuildType(Building* building);
    void resetBuildType(Building* building);
    bool insertGoldSlot(Building* building, Unit* unit);
    bool unloadGoldSlot(Building* building);

    void aFollowb(Unit* a, Entity* b);
    void stopFollow(Unit* unit);

    void insertProjectile(Projectile* p);
    bool removeProjectile(Projectile* p);

    bool executeCommand(std::string com, std::vector<Unit*> select);
    bool executeCommand(std::string com, Building* select);
    bool executeCommand(std::string com, std::vector<Unit*> select, float px, float py);
    bool executeCommand(std::string com, Building* select, float px, float py);


protected:
private:
    Map* map_;
    Graph* graph_;
    ALLEGRO_DISPLAY *display;
    ALLEGRO_TIMER *timer;
    ALLEGRO_EVENT_QUEUE *queue;
    std::vector<Unit*> unitSet_;
    std::vector<Building*> buildingSet_;
    std::vector<Projectile*> projectileSet_;
    std::vector<Player*> player_;
    Pathfinding* pf_;
    bool isInMapDimension(float px, float py);
    bool isPathBlocked(std::vector<std::pair<float,float>> pixelPath);
    void repath();
    void readConfig();
};

#endif // GAME_H

