#ifndef PLAYINGSTATE_H
#include <stack>
#include "State.h"
#include "Game.h"
#include "Graph.h"
#include "Entity.h"
#include "Unit.h"
#include "Building.h"
#include "Gui.h"
#include "nihgui.h"
#include "Mymath.h"
#include "UnitProperty.h"
#include "Collision.h"
#include "AI.h"
#define PLAYINGSTATE_H

/**
 * Estado playing do simulador
 */
class PlayingState : public State {
public:
    PlayingState(ALLEGRO_DISPLAY* display, ALLEGRO_TIMER* timer, ALLEGRO_EVENT_QUEUE* queue);
    static PlayingState* getInstance(ALLEGRO_DISPLAY* display, ALLEGRO_TIMER* timer, ALLEGRO_EVENT_QUEUE* queue) {
        if (instance == NULL) {
            instance = new PlayingState(display, timer, queue);
        }
        return instance;
    }
    virtual ~PlayingState();
    void init();
    void restartGame();
    void setGame(Game* game);
    void readInput(StateManager* sm);
    void update(StateManager* sm);
    void render(StateManager* sm);
    void executeCommand(std::string com);
    void runCommand(std::string com);
    void runCommand(std::string com, float px, float py);
    void changeState(int newState);

protected:
private:
    static PlayingState* instance;
    Game* game_;
    Map* map_;
    ALLEGRO_FONT* font_gui;
    Gui* gui;
    Theme* theme;
    int fps;
    int fps_accum;
    double fps_time;
    float selectSX;
    float selectSY;
    float selectFX;
    float selectFY;
    int mouse;
    float mousex, mousey;
    float zoom, rotate;
    float scroll_x, scroll_y;
    Collision* collision;
    void updateUnit();
    void updateCollision();
    void updateAttack();
    void updatePointer();
    void updateHp();
    void updateBuilding();
    void updateWorker();
    void updateIteration();
    void updateProjectile();
    Player* getWinner();

    void sharedInput(StateManager* sm, ALLEGRO_EVENT event);
    void idleInput(StateManager* sm, ALLEGRO_EVENT event);
    void selectUnitInput(StateManager* sm, ALLEGRO_EVENT event);
    void selectBuildingInput(StateManager* sm, ALLEGRO_EVENT event);
    void targetInput(StateManager* sm, ALLEGRO_EVENT event);

    void renderArrow(int px, int py, int x, int y, ALLEGRO_COLOR color);
    std::vector<Unit*> select;
    Building* selectBuilding;
    std::stack<int> action_state;
    int buildingID; // Só ta servindo pra renderizar a construcao no mouseover
    std::string command;

    AI** ai;
};

#endif // PLAYINGSTATE_H
