#ifndef GRAPH_H
#include <cstdio>
#include <cstdlib>
#include <memory>
#include <vector>
#include <map>
#include <set>
#include <string>
#include <algorithm>
#include <queue>
#include <cmath>
#include "Define.h"
#include "Edge.h"
#include "Node.h"
#include "TileProperty.h"
#include "BuildingProperty.h"
#define GRAPH_H

/**
 * Contem rotinas usando grafos
 */
class Graph {
public:
    Graph(std::vector<std::string> mapMatrix);
    virtual ~Graph();
    int getRows();
    int getColumns();
    Node*** getNode();
    Edge*** getAdj();
    std::map<int,int> getZoneEquivalency();
    void makeGraph(std::vector<std::string> mapMatrix);
    void insertEdge(Node* v, Node* w, float cost);
    void show();
    bool bfs(Node* s, Node* t);
    bool dijkstra(Node* s, Node* t);
    bool groupDijkstra(Node* s, std::vector<Node*> t);
    Node* noPathDijkstra(Node* s, std::set<int> zones);
    Node* noPathDijkstra2(Node* s, Node* t);
    Node* dijkstraBuild(Node* s, int buildType);
    Node* dijkstraEnemyBuilding(Node* s, int team);
    bool isInsertBuildingValid(Node* s, int buildType);
    bool isInMapDimension(int x, int y);
    double estimate(Node* s, Node* t);
    bool astar(Node* s, Node* t);
    void setZone();
    void setZone2();

protected:
private:
    int rows_;
    int columns_;
    Edge*** adj_;
    Node*** node_;
    std::map<int,int> zoneEquivalency_;
};

#endif // GRAPH_H
