#ifndef UNIT_H
#include <cmath>
#include <vector>
#include <string>
#include <cmath>
#include "Entity.h"
#include "Mymath.h"
#include "UnitProperty.h"
#include "BuildingProperty.h"
#include "Define.h"
#include <allegro5/allegro_primitives.h>
#define UNIT_H

/**
 * Classe de unidades
 */
class Unit : public Entity {
public:
    Unit(float px, float py, int type, int team);
    virtual ~Unit();

    float getTx();
    float getTy();
    std::pair<float,float> getT();
    int getSignal();
    bool isHold();
    bool isAgressive();
    float getCooldown();
    float getGroupRadius();
    Entity* getTarget();
    Entity* getFocus();
    Entity* getFollow();
    std::pair<float,float> getLastMove();
    std::vector<std::pair<float,float>> getCollisionPath();
    int getBuild();
    Entity* getGoldMine();

    void setTx(float tx);
    void setTy(float ty);
    void setT(float tx, float ty);
    void setSignal(int signal);
    void setHold(bool hold);
    void setAgressive(bool agressive);
    void setCooldown(float cooldown);
    void setGroupRadius(float groupRadius);
    void setTarget(Entity* target);
    void setFocus(Entity* focus);
    void setFollow(Entity* follow);
    void setLastMove(std::pair<float,float> lastMove);
    void setCollisionPath(std::vector<std::pair<float,float>> collisionPath);
    void setBuild(int build);
    void setGoldMine(Entity* goldMine);

    float getSpeed();
    float getRadius();
    float getRange();
    float getDamage();
    float getAttackSpeed();
    float getMaxHp();
    int getCost();
    int getProjectile();
    int getCount();
    float getAggroRadius();
    std::vector<ALLEGRO_BITMAP*> getBitmap();
    std::vector<std::string> getAction();
    std::string getName();
    float getTime();

    bool isMoving();
    bool inCollision();
    bool isAttackReady();

    void stop();
    void wait();
    void move(std::pair<float,float> d);
    void updateCooldown();
    void updateMove();
    void updateSpriteConfig();
    void update();
    //void setPath(std::vector<std::pair<int,int>> path);
    //std::vector<std::pair<int,int>> getPath();
    std::pair<float,float> getNextMovement();
    float getPixelPathDistance();
    void checkPath();
    int getNearestPath();
    void deleteFarPath();
    bool isUnit();
    void render();
    void actionSprite();

    int getXFollow();
    int getYFollow();
    void setXFollow(int x);
    void setYFollow(int y);
    void resetFollow();
    bool isRepath();
    void setRepath(bool repath);
    void resetCollision();
    Entity* collide;
    float dist;
    int ncollide;

    /* Auxilia o bot, usado somente para que a IA nao execute o mesmo comando de movimentacao todo frame
       Essa variavel e' modificada apenas no metodo unitMove da classe Game */
    std::pair<float,float> movePosition;

protected:
private:
    float tx_; /* Coordenada X do target da unidade */
    float ty_; /* Coordenada Y do target da unidade */
    float groupRadius_; /* Raio do grupo, serve para parar a movimentacao mais longe quanto maior o tamanho grupo */
    int signal_; /* Serve apenas para direcionar o sentido da colisao em hold */
    bool wait_; /* Espera na movimentacao se for true */
    bool pushed_; /* Se foi empurrada */
    bool agressive_; /* Se a unidade estiver no modo agressivo, ela ataca outras entidades inimigas que estao no seu alcance */
    float cooldown_; /* Cooldown do ataque */
    bool hold_; /* Se for true, ninguem pode empurrar */
    float movingTime_; /* Usado pra ver qual sprite de movimentacao renderizar */
    float actionTime_; /* Usado pra ver qual sprite de acao renderizar */
    bool renderAct_;
    Entity* target_;
    Entity* focus_;
    Entity* follow_;
    Entity* goldMine_;
    int xfollow;
    int yfollow;
    int build_;
    float angle_;
    std::pair<float,float> lastMove_;
    //std::vector<std::pair<int,int>> path_;
    std::vector<std::pair<float,float>> collisionPath_;
    bool repath_;

};

#endif // UNIT_H
