/* A hierarquia do codigo da IA foi baseada no bot chamado Nova, criado para BWAPI.
Site do Nova: http://nova.wolfwork.com/
*/

#include "AI.h"

/**
 * Construtor
 * @param game Ponteiro do game
 * @param player Ponteiro do player
 */
AI::AI(Game* game, Player* player) : game_(game), player_(player) {
    infoManager_ = new InfoManager(game,player);
    strategyManager_ = new StrategyManager(game,player,infoManager_);
    productionManager_ = new ProductionManager(game,player,infoManager_);
    buildManager_ = new BuildManager(game,player,infoManager_);
    squadManager_ = new SquadManager(game,player,infoManager_);
    workerManager_ = new WorkerManager(game,player,infoManager_);
    infoManager_->init();
}

/**
 * Destrutor
 */
AI::~AI() {
    printf("Deletando managers da AI\n");
    delete infoManager_;
    delete strategyManager_;
    delete productionManager_;
    delete buildManager_;
    delete squadManager_;
    delete workerManager_;
}

/**
 * @return Indice do time da IA
 */
int AI::getTeam() {
    return player_->getTeam();
}

/**
 * Executa as tarefas da IA
 */
void AI::run() {
    infoManager_->run();
    productionManager_->run();
    buildManager_->run();
    workerManager_->run();
    squadManager_->run();
}

/**
 * Insere o ponteiro unit na IA
 * @param unit Ponteiro da unidade
 */
void AI::insertUnit(Unit* unit) {
    if(getTeam() == unit->getTeam()) {
        infoManager_->insertUnit(unit);
        //printf("AI: inseriu unidade aliada\n");
    }
    else {
        //printf("AI: inseriu unidade inimiga\n");
    }
}

/**
 * Remove o ponteiro unit da IA
 * @param unit Ponteiro da unidade
 */
void AI::removeUnit(Unit* unit) {
    if(getTeam() == unit->getTeam()) {
        infoManager_->removeUnit(unit);
        //printf("AI: removeu unidade aliada\n");
    }
    else {
        //printf("AI: removeu unidade inimiga\n");
    }
}

/**
 * Insere o ponteiro building dentro da IA
 * @param building Ponteiro do edificio
 */
void AI::insertBuilding(Building* building) {
    if(getTeam() == building->getTeam()) {
        infoManager_->insertBuilding(building);
        //printf("AI: inseriu construcao aliada\n");
    }
    else {
        infoManager_->findNewAttackPoint = true;
        //printf("AI: inseriu construcao inimiga\n");
    }
}

/**
 * Remove o ponteiro unit dentro da IA
 * @param building Ponteiro do edificio
 */
void AI::removeBuilding(Building* building) {
    if(getTeam() == building->getTeam()) {
        infoManager_->removeBuilding(building);
        //printf("AI: removeu construcao aliada\n");
    }
    else {
        infoManager_->findNewAttackPoint = true;
        //printf("AI: removeu construcao inimiga\n");
    }
}
