#include "BuildManager.h"

/**
 * Construtor
 * @param game Ponteiro do game
 * @param player Ponteiro do player
 * @param infoManager Ponteiro do infoManager
 */
BuildManager::BuildManager(Game* game, Player* player, InfoManager* infoManager) : Manager(game,player), infoManager_(infoManager) {

}

/**
 * Destrutor
 */
BuildManager::~BuildManager() {

}

/**
 * Executa a rotina de construcao de edificio
 */
void BuildManager::run() {
    if(infoManager_->buildOrder.size() > 0) {
        int buildType = infoManager_->buildOrder[0];
        std::pair<float,float> buildLocation = getBuildLocation(infoManager_->ourBase.first, infoManager_->ourBase.second, buildType);
        infoManager_->buildLocation = buildLocation;
        //Unit* worker = getNearestWorker(buildLocation.first, buildLocation.second);
        Unit* worker = infoManager_->worker;
        if(worker != NULL) {
            std::vector<Unit*> select;
            select.push_back(worker);
            if(game_->executeCommand(BuildingProperty::getInstance()->getName(buildType), select, buildLocation.first, buildLocation.second)) {
                infoManager_->buildOrder.erase(infoManager_->buildOrder.begin());
                infoManager_->worker = NULL;
            }
        }
    }
}

/**
 * Usa o algoritmo Dijkstra para encontrar o ponto (x,y) tal que seja possivel construir um edificio do tipo buildType
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @param buildType Tipo do edificio
 * @return Ponto (x,y) em pixel
 */
std::pair<float,float> BuildManager::getBuildLocation(float px, float py, int buildType) {
    Graph* graph = game_->getGraph();
    Node* s = graph->getNode()[mymath::div(px,TILE_WIDTH)][mymath::div(py,TILE_HEIGHT)];
    Node* newNode;
    if((newNode = graph->dijkstraBuild(s, buildType)) != NULL) {
        float newPx = (newNode->getX()+BuildingProperty::getInstance()->getDimension(buildType)/2)*TILE_WIDTH + TILE_WIDTH/2;
        float newPy = (newNode->getY()+BuildingProperty::getInstance()->getDimension(buildType)/2)*TILE_HEIGHT + TILE_HEIGHT/2;
        return std::make_pair(newPx, newPy);
    }
    return std::make_pair(-1,-1);
}
