#include "InfoManager.h"

/**
 * Construtor
 * @param game Ponteiro do game
 * @param player Ponteiro do player
 */
InfoManager::InfoManager(Game* game, Player* player) : Manager(game,player) {
    worker = NULL;
    findNewAttackPoint = false;

    int rows = game_->getMap()->getMapMatrix().size();
    int columns = game_->getMap()->getMapMatrix()[0].size();
	nodes = new MaxflowGraph::node_id*[rows];
	for(int i = 0; i < rows; i++) {
        nodes[i] = new MaxflowGraph::node_id[columns];
	}
	maxflowGraph = new MaxflowGraph();
	for(int i = 0; i < rows; i++) {
        for(int j = 0; j < columns; j++) {
            nodes[i][j] = maxflowGraph->add_node();
        }
	}
	makeGraph();
}

/**
 * Destrutor
 */
InfoManager::~InfoManager() {
    printf("Deletando nodes de InfoManager\n");
    int rows = game_->getMap()->getMapMatrix().size();
    for(int i = 0; i < rows; i++) {
        delete [] nodes[i];
    }
    delete [] nodes;
    delete maxflowGraph;
}

/**
 * Inicializa infoManager
 */
void InfoManager::init() {
    std::vector<Unit*> playerUnitSet = player_->getUnitSet();
    std::vector<Building*> playerBuildingSet = player_->getBuildingSet();
    std::vector<Building*> buildingSet = game_->getBuildingSet();
    for(int i = 0; i < (int) buildingSet.size(); i++) {
        Building* b = buildingSet[i];
        if(b->getType() == BUILDING_TYPE::HALL) {
            if(b->getTeam() == player_->getTeam()) {
                ourBase = std::make_pair(b->getPx(), b->getPy());
            } else {
                enemyBase = std::make_pair(b->getPx(), b->getPy());
            }
        }
    }
    //printf("OUR: %f %f, ENEMY: %f %f\n", ourBase.first, ourBase.second, enemyBase.first, enemyBase.second);
    for(int i = 0; i < (int) playerUnitSet.size(); i++) {
        Unit* u = playerUnitSet[i];
        insertUnit(u);
    }
    for(int i = 0; i < (int) playerBuildingSet.size(); i++) {
        Building* b = playerBuildingSet[i];
        insertBuilding(b);
    }
    //chokePoint = std::make_pair(18*16,14*16);
    barracksProduction = UNIT_TYPE::ARCHER;
    //if(player_->getTeam() == 0) barracksProduction = UNIT_TYPE::ARCHER;
    //else barracksProduction = UNIT_TYPE::WARRIOR;
	setTWeights();
	setChokePoint();
	setAttack();
}

/**
 * Insere o ponteiro unit na IA
 * @param unit Ponteiro da unidade
 * @return true se inseriu, false caso contrario
 */
bool InfoManager::insertUnit(Unit* unit) {
    std::vector<Unit*>::iterator it;
    it = find (ourUnitSet[unit->getType()].begin(), ourUnitSet[unit->getType()].end(), unit);
    if (it == ourUnitSet[unit->getType()].end()) {
        ourUnitSet[unit->getType()].push_back(unit);
        return true;
    }
    return false;

}

/**
 * Remove o ponteiro unit da IA
 * @param unit Ponteiro da unidade
 * @return true se removeu, false caso contrario
 */
bool InfoManager::removeUnit(Unit* unit) {
    std::vector<Unit*>::iterator it;
    it = find (ourUnitSet[unit->getType()].begin(), ourUnitSet[unit->getType()].end(), unit);
    if (it != ourUnitSet[unit->getType()].end()) {
        ourUnitSet[unit->getType()].erase(it);
        return true;
    }
    return false;
}

/**
 * Insere o ponteiro building dentro da IA
 * @param building Ponteiro do edificio
 * @return true se inseriu, false caso contrario
 */
bool InfoManager::insertBuilding(Building* building) {
    std::vector<Building*>::iterator it;
    it = find (ourBuildingSet[building->getType()].begin(), ourBuildingSet[building->getType()].end(), building);
    if (it == ourBuildingSet[building->getType()].end()) {
        ourBuildingSet[building->getType()].push_back(building);
        return true;
    }
    return false;
}

/**
 * Remove o ponteiro unit dentro da IA
 * @param building Ponteiro do edificio
 * @return true se removeu, false caso contrario
 */
bool InfoManager::removeBuilding(Building* building) {
    std::vector<Building*>::iterator it;
    it = find (ourBuildingSet[building->getType()].begin(), ourBuildingSet[building->getType()].end(), building);
    if (it != ourBuildingSet[building->getType()].end()) {
        ourBuildingSet[building->getType()].erase(it);
        return true;
    }
    return false;
}

/**
 * Insere buildType no buildOrder
 * @param buildType Tipo do edificio
 * @return true se inseriu, false caso contrario
 */
bool InfoManager::requestBuilding(int buildType) {
    std::vector<int>::iterator it;
    it = find(buildOrder.begin(), buildOrder.end(), buildType);
    if(it == buildOrder.end()) {
        //printf("%d foi inserida em buildOrder\n", buildType);
        buildOrder.push_back(buildType);
        return true;
    }
    return false;
}

/**
 * Executa rotina de informacoes
 */
void InfoManager::run() {
    if(findNewAttackPoint) {
        setAttack();
        findNewAttackPoint = false;
    }
}

/**
 * Gera o grafo de acordo com o mapa, usado para o maxflow
 */
void InfoManager::makeGraph() {
    int i, j;
    std::vector<std::string> mapMatrix = game_->getMap()->getMapMatrix();
    int rows = mapMatrix.size();
    int columns = mapMatrix[0].size();
	for(i = 0; i < rows; i++) {
		for(j = 0; j < columns; j++) {
            int type = TileProperty::getInstance()->getType(mapMatrix[i][j]);
            if(TileProperty::getInstance()->isWalkable(type)) {
                if(j+1 < columns) {
                    int type2 = TileProperty::getInstance()->getType(mapMatrix[i][j+1]);
                    if(TileProperty::getInstance()->isWalkable(type2)) {
                        maxflowGraph->add_edge(nodes[i][j], nodes[i][j+1], 1, 1);
                    }
                }
                if(i+1 < rows) {
                    int type2 = TileProperty::getInstance()->getType(mapMatrix[i+1][j]);
                    if(TileProperty::getInstance()->isWalkable(type2)) {
                        maxflowGraph->add_edge(nodes[i][j], nodes[i+1][j], 1, 1);
                    }

                }
                // diagonais, talvez esteja errado o peso das diagonais
                if(i+1 < rows && j+1 < columns) {
                    int type2 = TileProperty::getInstance()->getType(mapMatrix[i+1][j+1]);
                    if(TileProperty::getInstance()->isWalkable(type2)) {
                        maxflowGraph->add_edge(nodes[i][j], nodes[i+1][j+1], sqrt(2), sqrt(2));
                    }
                }
                if(i+1 < rows && j-1 >= 0) {
                    int type2 = TileProperty::getInstance()->getType(mapMatrix[i+1][j-1]);
                    if(TileProperty::getInstance()->isWalkable(type2)) {
                        maxflowGraph->add_edge(nodes[i][j], nodes[i+1][j-1], sqrt(2), sqrt(2));
                    }
                }
            }
		}
	}
}

/**
 * Atribui o valor do fluxo de entrada e saida dos nos que fazem parte do edifcio principal aliado e inimigo
 */
void InfoManager::setTWeights() {
    int i, j, n;
    std::vector<Building*> buildingSet = game_->getBuildingSet();
    for(n = 0; n < (int) buildingSet.size(); n++) {
        Building* b = buildingSet[n];
        if(b->getType() == BUILDING_TYPE::HALL) {
            if(b->getTeam() == player_->getTeam()) {
                for(i = 0; i < b->getDimension(); i++) {
                    for(j = 0; j < b->getDimension(); j++) {
                        maxflowGraph->set_tweights(nodes[b->getX()+i][b->getY()+j], INF, 0);
                    }
                }
            } else {
                for(i = 0; i < b->getDimension(); i++) {
                    for(j = 0; j < b->getDimension(); j++) {
                        maxflowGraph->set_tweights(nodes[b->getX()+i][b->getY()+j], 0, INF);
                    }
                }
            }
        }
    }
}

/**
 * Atribui o ponto de estrangulamento da IA encontrado pelo algoritmo maxflow
 * Escolhemos o no' medio para ser o ponto de estrangulamento
 */
void InfoManager::setChokePoint() {
    int i, j;
    std::vector<std::string> mapMatrix = game_->getMap()->getMapMatrix();
    int columns = mapMatrix[0].size();
    int rows = mapMatrix.size();
    std::vector<std::pair<int,int>> choke_points;

    maxflowGraph->maxflow();
	//MaxflowGraph::flowtype flow = maxflowGraph->maxflow();
	//printf("Flow = %d\n", flow);
    Graph* G = game_->getGraph();
    Edge* p;
    Node* w;
    for(i = 0; i < rows; i++) {
        for(j = 0; j < columns; j++) {
            // Se o no' atual estiver na area SOURCE
            if(maxflowGraph->what_segment(nodes[i][j]) == MaxflowGraph::SOURCE) {
                // Verifica os vizinhos do no' atual
                for(p = G->getAdj()[i][j]; p != NULL; p = p->getNext()) {
                    w = p->getNode();
                    // Insere no vetor choke_points somente se o vizinho for andavel e estiver na area SINK
                    if(w->isWalkable() && maxflowGraph->what_segment(nodes[w->getX()][w->getY()]) == MaxflowGraph::SINK) {
                        choke_points.push_back(std::make_pair(i,j));
                        break;
                    }
                }
            }
        }
    }
    for(int i = 0; i < (int) choke_points.size(); i++) {
        //printf("%d %d\n", choke_points[i].first, choke_points[i].second);
        G->getNode()[choke_points[i].first][choke_points[i].second]->setChoke(player_->getTeam());
    }
    //printf("CHOKE do meio: %d %d\n", choke_points[choke_points.size()/2].first, choke_points[choke_points.size()/2].second);
    //printf("CHOKE do meio em FLOAT: %f %f\n", choke_points[choke_points.size()/2].first*16.0, choke_points[choke_points.size()/2].second*16.0);
    chokePoint = std::make_pair(choke_points[choke_points.size()/2].first*16.0, choke_points[choke_points.size()/2].second*16.0);
}

/**
 * Atribui o ponto de ataque do micro(ponto do edificio inimigo mais proximo da base)
 */
void InfoManager::setAttack() {

    float px = ourBase.first;
    float py = ourBase.second;
    Graph* graph = game_->getGraph();
    Node* s = graph->getNode()[mymath::div(px,TILE_WIDTH)][mymath::div(py,TILE_HEIGHT)];
    Node* target;
    if((target = graph->dijkstraEnemyBuilding(s, player_->getTeam())) != NULL) {
        float targetPx = target->getX()*TILE_WIDTH + TILE_WIDTH/2;
        float targetPy = target->getY()*TILE_HEIGHT + TILE_HEIGHT/2;
        attackPoint = std::make_pair(targetPx, targetPy);
    }
    else {
        attackPoint = std::make_pair(enemyBase.first, enemyBase.second);
    }

}

