#include "WorkerManager.h"

/**
 * Construtor
 * @param game Ponteiro do game
 * @param player Ponteiro do player
 * @param infoManager Ponteiro do infoManager
 */
WorkerManager::WorkerManager(Game* game, Player* player, InfoManager* infoManager) : Manager(game,player), infoManager_(infoManager) {

}

/**
 * Destrutor
 */
WorkerManager::~WorkerManager() {

}

/**
 * Executa rotina de gerenciamento de unidades trabalhadoraas
 */
void WorkerManager::run() {
    // Se houver alguma requisicao de contrucao de edificio
    if(infoManager_->buildOrder.size() > 0) {
        Unit* worker = getIdleWorker();
        infoManager_->worker = worker;
        if(worker == NULL) {
            // Procura por alguma mina de ouro que possui unidade trabalhadora
            for(int i = 0; i < (int) infoManager_->ourBuildingSet[BUILDING_TYPE::GOLD_MINE].size(); i++) {
                Building *b = infoManager_->ourBuildingSet[BUILDING_TYPE::GOLD_MINE][i];
                if(b->getSlot().size() > 0) {
                    game_->executeCommand("Rally", b, infoManager_->buildLocation.first, infoManager_->buildLocation.second);
                    game_->executeCommand("Unload", b);
                    break;
                }
            }
        }
    }
    // Envia cada unidade trabalhadora ociosa a alguma mina de ouro com espaco
    else {
        std::vector<Unit*> select;
        Building* gm = NULL;
        for(int i = 0; i < (int) infoManager_->ourUnitSet[UNIT_TYPE::WORKER].size(); i++) {
            Unit* worker = infoManager_->ourUnitSet[UNIT_TYPE::WORKER][i];
            for(int j = 0; j < (int) infoManager_->ourBuildingSet[BUILDING_TYPE::GOLD_MINE].size(); j++) {
                gm = infoManager_->ourBuildingSet[BUILDING_TYPE::GOLD_MINE][j];
                if((int) gm->getSlot().size() < BuildingProperty::getInstance()->getMaxSlot(BUILDING_TYPE::GOLD_MINE)) {
                    break;
                }
            }
            if(worker->getGoldMine() == NULL && worker->getBuild() == -1 && worker->getEntity() == NULL) {
                select.push_back(worker);
            }
        }
        if(gm != NULL) {
            game_->executeCommand("Gather", select, gm->getPx(), gm->getPy());
        }
    }
}

/**
 * Seleciona uma unidade trabalhadora ociosa
 * @return Ponteiro da unidade trabalhadora, NULL se nao existe
 */
Unit* WorkerManager::getIdleWorker() {
    for(int i = 0; i < (int) infoManager_->ourUnitSet[UNIT_TYPE::WORKER].size(); i++) {
        Unit* worker = infoManager_->ourUnitSet[UNIT_TYPE::WORKER][i];
        if(worker->getGoldMine() == NULL && worker->getBuild() == -1 && worker->getEntity() == NULL) {
            return worker;
        }
    }
    return NULL;
}
