#include "Game.h"

/**
 * Construtor
 * @param map Ponteiro do mapa
 */
Game::Game(Map* map) : map_(map) {
    graph_ = new Graph(map_->getMapMatrix());
    pf_ = new Pathfinding(graph_);
    //insertUnit(new Unit(100,100,0,0));
    player_.push_back(new Player(0, 50, 60, al_map_rgba(0,0,255,64)));
    player_.push_back(new Player(1, 50, 60, al_map_rgba(255,0,0,64)));
    for(int i = 0; i < (int) player_.size(); i++) {
        player_[i]->setUnitCap(MAX_UNITCAP);
    }
    /*
    insertBuilding(new Building(3,7,BUILDING_TYPE::HALL, 0));
    insertBuilding(new Building(3,3,BUILDING_TYPE::GOLD_MINE, 0));
    insertBuilding(new Building(3,11,BUILDING_TYPE::BARRACKS, 0));
    insertBuilding(new Building(38,17,BUILDING_TYPE::HALL, 1));
    insertBuilding(new Building(38,21,BUILDING_TYPE::GOLD_MINE, 1));
    insertBuilding(new Building(38,13,BUILDING_TYPE::BARRACKS, 1));
    */
    readConfig();
    for(int i = 0; i < (int) buildingSet_.size(); i++) {
        buildingSet_[i]->setReady();
    }
}

/**
 * Destrutor
 */
Game::~Game() {
    printf("Deletando grafo, unitSet e buildingSet de Game\n");
    delete graph_;
    delete pf_;
    for(int i = 0; i < (int) unitSet_.size(); i++) {
        delete unitSet_[i];
    }
    for(int i = 0; i < (int) buildingSet_.size(); i++) {
        for(int j = 0; j < (int) buildingSet_[i]->getSlot().size(); j++) {
            delete buildingSet_[i]->getSlot()[j];
        }
        delete buildingSet_[i];
    }
    for(int i = 0; i < (int) player_.size(); i++) {
        delete player_[i];
    }

}

/**
 * Le configuracoes do mapa
 */
void Game::readConfig() {
    FILE* pFile = fopen(map_->getConfigName(), "r");
    char buildingName[20];
    int x;
    int y;
    int team;
    while(fscanf(pFile, "%s %d %d %d\n", buildingName, &x, &y, &team) > 0) {
        std::string name = buildingName;
        //printf("x: %d | y: %d | name: %s | team: %d\n", x, y, name.c_str(), team);
        insertBuilding(new Building(x,y,BuildingProperty::getInstance()->getType(name),team));
    }
    fclose(pFile);
}

/**
 * @return Vetor de unidades dentro do jogo
 */
std::vector<Unit*> Game::getUnitSet() {
    return unitSet_;
}

/**
 * Atribui vetor de unidades do jogo
 * @param unitSet Vetor de unidades
 */
void Game::setUnitSet(std::vector<Unit*> unitSet) {
    unitSet_ = unitSet;
}

/**
 * @return Vetor de edificios dentro do jogo
 */
std::vector<Building*> Game::getBuildingSet() {
    return buildingSet_;
}

/**
 * Atribui vetor de edificios
 * @param buildingSet Vetor de edificios
 */
void Game::setBuildingSet(std::vector<Building*> buildingSet) {
    buildingSet_ = buildingSet;
}

/**
 * @return Vetor de projeteis ativos
 */
std::vector<Projectile*> Game::getProjectileSet() {
    return projectileSet_;
}

/**
 * Atribui vetor de projeteis ativos
 * @param projectileSet Vetor de projeteis
 */
void Game::setProjectileSet(std::vector<Projectile*> projectileSet) {
    projectileSet_ = projectileSet;
}

/**
 * @return Vetor de jogadores
 */
std::vector<Player*> Game::getPlayer() {
    return player_;
}

/**
 * @return Grafo do grid
 */
Graph* Game::getGraph() {
    return graph_;
}

/**
 * Atribui grafo do grid no jogo
 * @param graph Grafo do grid
 */
void Game::setGraph(Graph* graph) {
    graph_ = graph;
}

/**
 * @return Mapa do jogo
 */
Map* Game::getMap() {
    return map_;
}

/**
 * Atribui mapa ao jogo
 * @param map Mapa
 */
void Game::setMap(Map* map) {
    map_ = map;
}

/**
 * Insere projetil no jogo
 * @param p Ponteiro do projetil
 */
void Game::insertProjectile(Projectile* p) {
    projectileSet_.push_back(p);
}

/**
 * Remove projetil do jogo
 * @param p Ponteiro do projetil
 * @return true se removeu, false caso contrario
 */
bool Game::removeProjectile(Projectile* p) {
    std::vector<Projectile*>::iterator it;
    it = find (projectileSet_.begin(), projectileSet_.end(), p);
    if (it != projectileSet_.end()) {
        projectileSet_.erase(it);
        delete p;
        return true;
    }
    return false;
}

/**
 * Insere unidade no jogo
 * @param unit Ponteiro da unidade
 */
void Game::insertUnit(Unit* unit) {
    unitSet_.push_back(unit);
    player_[unit->getTeam()]->insertUnit(unit);
}

// Nao esta sendo utilizado
/**
 * Remove unidade do jogo
 * @param unit Ponteiro da unidade
 * @return true se removeu, false caso contrario
 */
bool Game::removeUnit(Unit* unit) {
    int team = unit->getTeam();
    std::vector<Unit*>::iterator it;
    it = find (unitSet_.begin(), unitSet_.end(), unit);
    player_[team]->removeUnit(unit);
    player_[team]->sumUnitCount(-unit->getCount());
    if (it != unitSet_.end()) {
        unitSet_.erase(it);
        delete unit;
        return true;
    }
    return false;
}

/**
 * Verifica se da pra inserir um edificio do tipo buildType no ponto (px,py)
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @param buildType Tipo do edificio
 * @return true se der pra inserir um edificio do tipo buildType na posicao (px,py) do mapa
 */
bool Game::isInsertBuildingValid(float px, float py, int buildType) {
    BuildingProperty* bp = BuildingProperty::getInstance();
    int x = mymath::div(px,TILE_WIDTH);
    int y = mymath::div(py,TILE_HEIGHT);
    x = x - bp->getDimension(buildType)/2;
    y = y - bp->getDimension(buildType)/2;
    int dimension = BuildingProperty::getInstance()->getDimension(buildType);
    Node*** node = graph_->getNode();
    for(int i = x; i < x+dimension; i++) {
        for(int j = y; j < y+dimension; j++) {
            if(!isInMapDimension(i*TILE_WIDTH,j*TILE_HEIGHT) || BuildingProperty::getInstance()->getTileType(buildType) != node[i][j]->getTileType()) {
                return false;
            }
        }
    }
    return true;
}

/**
 * Verifica se ha alguma unidade na posicao (x,y) dentro das dimensoes da construcao do tipo buildType.
 * Serve para nao permitir construcao de edificios em cima de unidades
 * @param x
 * @param y
 * @param buildType
 * @return true se houver alguma unidade no ponto (x,y) considerando as dimensoes do edificio do tipo buildType, false caso contrario
 */
bool Game::hasUnitInBuilding(int x, int y, int buildType) {
    int dim = BuildingProperty::getInstance()->getDimension(buildType);
    for(int i = 0; i < (int) unitSet_.size(); i++) {
        Unit* u = unitSet_[i];
        if(x*TILE_WIDTH <= u->getPx() && u->getPx() <= (x+dim)*TILE_WIDTH) {
            if(y*TILE_HEIGHT <= u->getPy() && u->getPy() <= (y+dim)*TILE_HEIGHT) {
                return true;
            }
        }
    }
    return false;
}

/**
 * Verifica se um vetor de caminhos em pixel esta bloqueado por algum edificio
 * @param pixelPath Caminho em pixel
 * @return true se o caminho estiver bloqueado, false caso contrario
 */
bool Game::isPathBlocked(std::vector<std::pair<float,float>> pixelPath) {
    Node*** node = graph_->getNode();
    for(int i = 0; i < (int) pixelPath.size(); i++) {
        int x = mymath::div(pixelPath[i].first, TILE_WIDTH);
        int y = mymath::div(pixelPath[i].second,TILE_HEIGHT);
        if(!node[x][y]->isWalkable()) return true;
    }
    return false;
}

/**
 * Pra cada unidade e construção verifica se o caminho deles foi bloqueado.
 * Em caso afirmativo, refaz pathfinding
 */
void Game::repath() {
    for(int i = 0; i < (int) unitSet_.size(); i++) {
        std::vector<std::pair<float,float>> pixelPath = unitSet_[i]->getPixelPath();
        if(isPathBlocked(pixelPath)) {
            float tx = pixelPath[pixelPath.size()-1].first;
            float ty = pixelPath[pixelPath.size()-1].second;
            unitMove(unitSet_[i], tx, ty);
        }
    }
    for(int i = 0; i < (int) buildingSet_.size(); i++) {
        std::vector<std::pair<float,float>> pixelPath = buildingSet_[i]->getPixelPath();
        if(isPathBlocked(pixelPath)) {
            //printf("Bloqueado\n");
            float tx = pixelPath[pixelPath.size()-1].first;
            float ty = pixelPath[pixelPath.size()-1].second;
            //printf("Buscando outro caminho para %f %f\n", tx,ty);
            buildingPath(buildingSet_[i], tx,ty);
        }
    }
}

/**
 * Insere edificio no jogo
 * @param building Ponteiro do edificio
 */
void Game::insertBuilding(Building* building) {
    int dimension = building->getDimension();
    //Node*** node = graph_->getNode();
    std::vector<std::pair<float,float>> pixelPath;
    pixelPath.push_back(std::make_pair(building->getPx() - (1+(float)((dimension-1)/2))*TILE_WIDTH, building->getPy() + (1+(float)((dimension-1)/2))*TILE_HEIGHT));
    //printf("%f\n", )
    building->setPixelPath(pixelPath);
    //setBuildingWalkable(building,false);
    setBuildType(building);
    buildingSet_.push_back(building);
    player_[building->getTeam()]->insertBuilding(building);
    graph_->setZone();
    repath();
}


// Nao esta sendo usado
/**
 * Remove edificio do jogo
 * @param building Ponteiro do edificio
 * @return true se removeu, false caso contrario
 */
bool Game::removeBuilding(Building* building) {
    std::vector<Building*>::iterator it;
    it = find (buildingSet_.begin(), buildingSet_.end(), building);
    player_[building->getTeam()]->removeBuilding(building);
    int team = building->getTeam();
    if (it != buildingSet_.end()) {
        for(int i = 0; i < (int) building->getSlot().size(); i++) {
            removeUnit(building->getSlot()[i]);
        }
        for(int i = 0; i < (int) building->getQueue().size(); i++) {
            player_[team]->sumUnitCount(-UnitProperty::getInstance()->getCount(building->getQueue()[i]));
        }
        buildingSet_.erase(it);
        //setBuildingWalkable(building, true);
        resetBuildType(building);
        graph_->setZone();
        delete building;
        return true;
    }
    return false;
}

/**
 * Atribui as caracteriscas dos tiles correspondentes ao edificio
 * @param building Ponteiro do edificio
 */
void Game::setBuildType(Building* building) {
    int x = building->getX();
    int y = building->getY();
    int dimension = building->getDimension();
    Node*** node = graph_->getNode();
    for(int i = x-1; i < x+dimension+1; i++) {
        for(int j = y-1; j < y+dimension+1; j++) {
            if(i >= 0 && i < graph_->getRows() && j >= 0 && j < graph_->getColumns()) {
                if(node[i][j]->getTileType() >= 0) node[i][j]->setTileType(-1);
                else node[i][j]->setTileType(node[i][j]->getTileType()-1);
                if(i >= x && i < x+dimension && j >=y && j < y+dimension) {
                    node[i][j]->setWalkable(false);
                    node[i][j]->setTeam(building->getTeam());
                }
            }
        }
    }
}

/**
 * Remove as caracteriscas dos tiles correspondentes ao edificio
 * @param building Ponteiro do edificio
 */
void Game::resetBuildType(Building* building) {
    int x = building->getX();
    int y = building->getY();
    int dimension = building->getDimension();
    Node*** node = graph_->getNode();
    for(int i = x-1; i < x+dimension+1; i++) {
        for(int j = y-1; j < y+dimension+1; j++) {
            if(i >= 0 && i < graph_->getRows() && j >= 0 && j < graph_->getColumns()) {
                if(node[i][j]->getTileType() == -1) node[i][j]->setTileType(TileProperty::getInstance()->getTileType(node[i][j]->getType()));
                else node[i][j]->setTileType(node[i][j]->getTileType()+1);
                if(i >= x && i < x+dimension && j >=y && j < y+dimension) {
                    node[i][j]->setWalkable(TileProperty::getInstance()->isWalkable(node[i][j]->getType()));
                    node[i][j]->setTeam(-1);
                }
            }
        }
    }
}

/**
 * Atribui aos tiles correspondentes ao edificio a caracteristica walkable
 * @param building Ponteiro do edificio
 * @param boolean Valor de walkable
 * @return true se building nao for nulo, false caso contrario
 */
bool Game::setBuildingWalkable(Building* building, bool boolean) {
    if(building == NULL) return false;
    int x = building->getX();
    int y = building->getY();
    int dimension = building->getDimension();
    Node*** node = graph_->getNode();
    for(int i = x; i < x+dimension; i++) {
        for(int j = y; j < y+dimension; j++) {
            node[i][j]->setWalkable(boolean);
        }
    }
    return true;
}

/**
 * Busca um caminho do centro do edificio ao ponto (px,py) e o atribui como caminho
 * @param building Ponteiro do edificio
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 */
bool Game::buildingPath(Building* building, float px, float py) {
    std::set<int> ze1 = pf_->getZE(building);
    Building* b = NULL;
    std::set<int> ze2;
    std::vector<std::pair<float,float>> pixelPath;
    Entity* entity = getEntity(px,py);
    if(isInMapDimension(px,py)) {
        building->setEntity(NULL);
        // Se o ponto (px,py) tiver um edificio
        if(entity != NULL && entity->isBuilding()) {
            b = (Building*) entity;
            ze2 = pf_->getZE(b);
            if(!pf_->isInSameZE(ze1,ze2)) {
                std::pair<float,float> newT = pf_->findNearestTile(px,py,ze1);
                px = newT.first;
                py = newT.second;

            } else {
                px = b->getPx();
                py = b->getPy();
            }
            building->setEntity(b);
        }
        // Se o ponto (px,py) nao estiver na mesma ZE de building
        else if(!pf_->isInSameZE(pf_->getZE(px,py),ze1)) {
            std::pair<float,float> newT = pf_->findNearestTile(px,py,ze1);
            px = newT.first;
            py = newT.second;

        }
        setBuildingWalkable(building, true);
        setBuildingWalkable(b, true);

        pixelPath = pf_->astarPathfinding(building->getPx(),building->getPy(),px,py);

        setBuildingWalkable(building, false);
        setBuildingWalkable(b, false);
        if(pixelPath.size() > 0) {
            pixelPath = pf_->getWalkingPath(pixelPath);
            building->setPixelPath(pixelPath);
        }

    }
    return pixelPath.size() > 0;
}

/**
 * Faz unidade unit mover para o ponto (px,py) do mapa
 * @param unit Ponteiro da unidade
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @return true se existe um caminho entre a unidade e o ponto (px,py)
 */
bool Game::unitMove(Unit* unit, float px, float py) {
    Node*** node = graph_->getNode();
    int posX = mymath::div(unit->getPx(), TILE_WIDTH);
    int posY = mymath::div(unit->getPy(), TILE_HEIGHT);
    int dx = 0;
    int dy = 0;
    Building* b = NULL;
    std::set<int> ze1 = pf_->getZE(unit);
    std::set<int> ze2;
    Entity* entity = getEntity(px,py);

    if(isInMapDimension(px,py)) {
        // Pra unidade conseguir se mover quando ele esta em um local com ZE = 0, ta ruim ainda(se tiver nao tiver nenhum tile livre em volta da unidade, ele ainda trava)
        if(node[posX][posY]->getZone() == 0) {
            for(int i = -1; i < 2; i++) {
                for(int j = -1; j < 2; j++) {
                    posX = (int) ((unit->getPx()+i*unit->getRadius())/TILE_WIDTH);
                    posY = (int) ((unit->getPy()+j*unit->getRadius())/TILE_HEIGHT);
                    if(posX >= 0 && posX < map_->getRows() && posY >= 0 && posY < map_->getColumns()) continue;
                    printf("%d %d\n", posX, posY);
                    if(graph_->getNode()[posX][posY]->getZone() != 0) {
                        dx = i;
                        dy = j;
                    }

                }
            }
        } else {
            if(entity != NULL && entity->isBuilding()) {
                b = (Building*) entity;
                ze2 = pf_->getZE(b);
                if(!pf_->isInSameZE(ze1,ze2)) {
                    std::pair<float,float> newT = pf_->findNearestTile(px,py,ze1);
                    px = newT.first;
                    py = newT.second;

                } else {
                    px = b->getPx();
                    py = b->getPy();
                }
            }
            else if(!pf_->isInSameZE(pf_->getZE(px,py),ze1)) {
                std::pair<float,float> newT = pf_->findNearestTile(px,py,ze1);
                px = newT.first;
                py = newT.second;
            }
        }
        setBuildingWalkable(b, true);

        std::vector<std::pair<float,float>> pixelPath = pf_->astarPathfinding(unit->getPx()+dx*TILE_WIDTH,unit->getPy()+dy*TILE_HEIGHT,px,py);

        setBuildingWalkable(b, false);
        if(pixelPath.size() > 0) {
            pixelPath = pf_->getWalkingPath(pixelPath);
            unit->resetCollision();
            unit->setPixelPath(pixelPath);
            return true;
        }
    }
    return false;
}

/**
 * Realiza acao de parar em um vetor de unidades
 * @param select Vetor de unidades
 * @return true
 */
bool Game::selectStop(std::vector<Unit*> select) {
    for(int i = 0; i < (int) select.size(); i++) {
        Unit* u = select[i];
        u->stop();
        if(u->getBuild() != -1) removeUnitBuild(u);
    }
    return true;
}

/**
 * Realiza acao de mover a um ponto (px,py) em um vetor de unidades
 * @param select Vetor de unidades
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @return true
 */
bool Game::selectMove(std::vector<Unit*> select, float px, float py) {
    for(int i = 0; i < (int) select.size(); i++) {
        Unit* u = select[i];
        u->setFocus(NULL);
        u->setEntity(NULL);
        u->setAgressive(false);
        u->resetFollow();
        if(u->getBuild() != -1) removeUnitBuild(u);
        // Seta raio do grupo, qdo chegar perto desse raio, a unidade se conforma e para
        if((select.size()-1) % 6 == 0) {
            select[i]->setGroupRadius(((select.size()-1)/6)*select[0]->getRadius());
        } else {
            select[i]->setGroupRadius((((select.size()-1)/6) + 1)*select[0]->getRadius());
        }
        if(unitMove(select[i],px,py)) {
            std::vector<std::pair<float,float>> pixelPath = select[i]->getPixelPath();
            if(pixelPath.size() > 0) {
                float tx = pixelPath[pixelPath.size()-1].first;
                float ty = pixelPath[pixelPath.size()-1].second;
                select[i]->setT(tx,ty);
            }
            select[i]->movePosition = std::make_pair(px,py);
        }
        /*
        unitMove(select[i], px, py);
        std::vector<std::pair<float,float>> pixelPath = select[i]->getPixelPath();
        if(pixelPath.size() > 0) {
            float tx = pixelPath[pixelPath.size()-1].first;
            float ty = pixelPath[pixelPath.size()-1].second;
            select[i]->setT(tx,ty);
        }
        */
    }
    return true;
}

/**
 * Realiza acao de mover agressivamente a um ponto (px,py) em um vetor de unidades
 * @param select Vetor de unidades
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @return true
 */
bool Game::selectMoveAttack(std::vector<Unit*> select, float px, float py) {
    /* Tira os comentarios pra permitir focus fire */
    //Entity* focus = getEntity(x,y);
    for(int i = 0; i < (int) select.size(); i++) {
        Unit* u = select[i];
        u->resetFollow();
        u->setFocus(NULL);
        //u->setFocus(focus);
        //u->setFollow(focus);
        u->setEntity(NULL);
        u->setAgressive(true);
        if(u->getBuild() != -1) removeUnitBuild(u);
        // Seta raio do grupo, qdo chegar perto desse raio, a unidade se conforma e para
        if((select.size()-1) % 6 == 0) {
            select[i]->setGroupRadius(((select.size()-1)/6)*select[0]->getRadius());
        } else {
            select[i]->setGroupRadius((((select.size()-1)/6) + 1)*select[0]->getRadius());
        }
        if(unitMove(select[i],px,py)) {
            std::vector<std::pair<float,float>> pixelPath = select[i]->getPixelPath();
            if(pixelPath.size() > 0) {
                float tx = pixelPath[pixelPath.size()-1].first;
                float ty = pixelPath[pixelPath.size()-1].second;
                select[i]->setT(tx,ty);
            }
            select[i]->movePosition = std::make_pair(px,py);
        }
        /*
        unitMove(select[i], px, py);
        std::vector<std::pair<float,float>> pixelPath = select[i]->getPixelPath();
        if(pixelPath.size() > 0) {
            float tx = pixelPath[pixelPath.size()-1].first;
            float ty = pixelPath[pixelPath.size()-1].second;
            select[i]->setT(tx,ty);
        }
        */
    }
    return true;
}

// Procura pelo worker desocupado selecionado mais proximo do ponto (x,y) e faz mover para este ponto com finalidade de construir um edificio do tipo build
// Daria pra usar o Dijkstra pra encontrar o worker mais proximo do ponto (x,y)
/**
 * Realiza acao de construir a um ponto (px,py) em um vetor de unidades
 * @param select Vetor de unidades
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @param buildType Tipo do edificio
 * @return true se encontrou uma unidade trabalhadora para construir, false caso contrario
 */
bool Game::selectMoveBuild(std::vector<Unit*> select, float px, float py, int buildType) {
    Unit* worker = NULL;
    std::vector<std::pair<float,float>> workerPath;
    int dist = INF;
    if(isInsertBuildingValid(px,py, buildType)) {
        for(int i = 0; i < (int) select.size(); i++) {
            Unit* u = select[i];
            // Escolhe um worker que nao possui nenhuma construcao pendente
            if(u->getType() == UNIT_TYPE::WORKER && u->getBuild() == -1) {
                if(pf_->isInSameZE(pf_->getZE(u), pf_->getZE(px,py))) {
                    Entity* entity = getEntity(u->getPx(),u->getPy());
                    Building* b = NULL;
                    if(entity != NULL && entity->isBuilding()) {
                        b = (Building*) entity;
                    }
                    if(b != NULL) setBuildingWalkable(b, true);
                    std::vector<std::pair<float,float>> pixelPath = pf_->astarPathfinding(u->getPx(),u->getPy(),px,py);
                    if(b != NULL) setBuildingWalkable(b, false);
                    if(pixelPath.size() > 0) {
                        // Escolhe o worker mais proximo
                        if(dist > (int) pixelPath.size()) {
                            dist = pixelPath.size();
                            worker = u;
                            workerPath = pixelPath;
                        }
                    }
                }
            }
        }
        if(worker != NULL && insertUnitBuild(worker,buildType)) {
            //worker->setPixelPath(workerPath);
            unitMove(worker,px,py);
            worker->setT(px,py);
            worker->setAgressive(false);
            worker->setFocus(NULL);
            worker->setEntity(NULL);
            return true;
        }
    }
    return false;
}

/**
 * Realiza acao de coleta de recurso a um ponto (px,py) em um vetor de unidades
 * @param select Vetor de unidades
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @return true se o ponto (px,py) possuir algum recurso, false caso contrario
 */
bool Game::selectMoveGather(std::vector<Unit*> select, float px, float py) {
    Entity* entity = getEntity(px,py);
    if(entity != NULL && entity->isBuilding()) {
        Building* b = (Building*) entity;
        if(b->getType() != BUILDING_TYPE::GOLD_MINE) {
            return false;
        }
    } else {
        return false;
    }
    for(int i = 0; i < (int) select.size(); i++) {
        Unit* u = select[i];
        if(u->getType() == UNIT_TYPE::WORKER && u->getTeam() == entity->getTeam()) {
            u->resetFollow();
            u->setFocus(NULL);
            u->setEntity(entity);
            u->setAgressive(false);
            if(u->getBuild() != -1) removeUnitBuild(u);

            unitMove(select[i],px,py);
            std::vector<std::pair<float,float>> pixelPath = select[i]->getPixelPath();
            if(pixelPath.size() > 0) {
                float tx = pixelPath[pixelPath.size()-1].first;
                float ty = pixelPath[pixelPath.size()-1].second;
                select[i]->setT(tx,ty);
                return true;
            }
        }
    }
    return false;
}

/**
 * Busca pela unidade inimiga mais proxima considerando o alcance da unidade e raio do alvo
 * @param unit Ponteiro da unidade
 * @return Ponteiro da unidade inimiga mais proxima
 */
Unit* Game::getEnemyUnit(Unit* unit) {
    Unit* u;
    Unit* target = NULL;
    float dist;
    float curDist = INF;
    for(int i = 0; i < (int) unitSet_.size(); i++) {
        u = unitSet_[i];
        if(u->getTeam() != unit->getTeam()) {
            dist = mymath::distance(u->getPx(), u->getPy(), unit->getPx(), unit->getPy());
            if(dist < curDist && u->getHp() > 0) {
                curDist = dist;// - unit->getRange() - u->getRadius();
                target = u;
            }
        }
    }
    return target;
}


/**
 * Busca pelo edificio inimigo mais proximo considerando o alcance da unidade e raio do alvo
 * @param unit Ponteiro da unidade
 * @return Ponteiro do edificio inimigo mais proximo
 */
Building* Game::getEnemyBuilding(Unit* unit) {
    Building* b;
    Building* target = NULL;
    float dist;
    float curDist = INF;
    for(int i = 0; i < (int) buildingSet_.size(); i++) {
        b = buildingSet_[i];
        if(b->getTeam() != unit->getTeam()) {
            dist = mymath::distance(b->getPx(), b->getPy(), unit->getPx(), unit->getPy());
            if(dist < curDist && b->getHp() > 0) {
                curDist = dist;// - unit->getRange() - b->getRadius();
                target = b;
            }
        }
    }
    return target;
}

/**
 * Verifica se o ponto (px,py) esta nas dimensoes do mapa
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @return true se (px,py) esta nas dimensoes do mapa, false caso contrario
 */
bool Game::isInMapDimension(float px, float py) {
    int posX = mymath::div(px,TILE_WIDTH);
    int posY = mymath::div(py,TILE_HEIGHT);
    if(posX >= 0 && posY >= 0 && posX < map_->getRows() && posY < map_->getColumns()) {
        //printf("pos %d %d map %d %d\n", posX, posY, map_->getRows(), map_->getColumns());
        return true;
    }
    return false;
}

/**
 * Insere requisicao de treinamento de unidade do tipo unitType a um edificio building
 * @param building Ponteiro do edificio
 * @param unitType Tipo da unidade
 * @return true se inseriu corretamente, false caso contrario
 */
bool Game::insertBuildingQueue(Building* building, int unitType) {
    if(building->isActive()) {
        int team = building->getTeam();
        int teamGold = player_[team]->getGold();
        int cost = UnitProperty::getInstance()->getCost(unitType);
        int count = UnitProperty::getInstance()->getCount(unitType);
        int unitCount = player_[team]->getUnitCount();
        int unitCap = player_[team]->getUnitCap();
        if(teamGold >= cost && unitCount + count <= unitCap && (int) building->getQueue().size() < BuildingProperty::getInstance()->getMaxQueue(building->getType())) {
            building->insertQueue(unitType);
            player_[team]->sumGold(-cost);
            player_[team]->sumUnitCount(count);
            return true;
        }
    }
    return false;
}

/**
 * Cancela a primeira unidade na fila de treinamento do edificio building
 * @param building Ponteiro do edificio
 * @return true se cancelou corretamente, false caso contrario
 */
bool Game::cancelBuildingQueue(Building* building) {
    if(building->getQueue().size() > 0) {
        int team = building->getTeam();
        int buildType = building->getQueue().back();
        int cost = UnitProperty::getInstance()->getCost(buildType);
        int count = UnitProperty::getInstance()->getCount(buildType);
        building->removeQueue(building->getQueue().size()-1);
        player_[team]->sumGold(cost);
        player_[team]->sumUnitCount(-count);
        return true;
    }
    return false;
}

/**
 * Insere um edificio do tipo buildType a uma unidade unit para ser construida
 * @param unit Ponteiro da unidade
 * @param buildType Tipo do edificio
 * @return true se inseriu corretamente, false caso contrario
 */
bool Game::insertUnitBuild(Unit* unit, int buildType) {
    if(unit->getType() == UNIT_TYPE::WORKER) {
        int teamGold = player_[unit->getTeam()]->getGold();
        int cost = BuildingProperty::getInstance()->getCost(buildType);
        if(teamGold >= cost) {
            unit->setBuild(buildType);
            player_[unit->getTeam()]->sumGold(-cost);
            return true;
        }
    }
    return false;
}

/**
 * Cancela o edificio que a unidade unit iria construir
 * @param unit Ponteiro da unidade
 * @return true se cancelou corretamente, false caso contrario
 */
bool Game::removeUnitBuild(Unit* unit) {
    if(unit->getType() == UNIT_TYPE::WORKER) {
        if(unit->getBuild() != -1) {
            int cost = BuildingProperty::getInstance()->getCost(unit->getBuild());
            unit->setBuild(-1);
            player_[unit->getTeam()]->sumGold(cost);
            return true;
        }
    }
    return false;
}

// Se duas entidade estiverem sobrepostos, prioriza construcao
/**
 * Devolve o ponteiro da entidade na posicao (px,py)
 * @return Ponteiro da entidade na posicao (px,py)
 */
Entity* Game::getEntity(float px, float py) {
    for(int i = 0; i < (int) buildingSet_.size(); i++) {
        Building* b = buildingSet_[i];
        if(mymath::distance(b->getPx(), b->getPy(), px, py) <= b->getRadius()) {
            return b;
        }
    }
    for(int i = 0; i < (int) unitSet_.size(); i++) {
        Unit* u = unitSet_[i];
        if(mymath::distance(u->getPx(), u->getPy(), px, py) <= u->getRadius()) {
            return u;
        }
    }
    return NULL;
}

// O mouse == 2 no ponto (x,y) tem diferentes funções quando clicada em cima de uma entidade:
// 1) Quando qualquer unidade é clicada em cima de outra entidade inimiga, seta agressive true e focus = entity
// 2) Quando worker é clicada em cima de uma entidade aliada, seta entitidade do worker pra entity
/**
 * Realiza acao multipla a um vetor de unidades no ponto (px,py)
 * @param select Vetor de unidades
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 */
void Game::multiAction(std::vector<Unit*> select, float px, float py) {
    Entity* entity = getEntity(px,py);
    selectMove(select,px,py);
    if(entity != NULL) {
        for(int i = 0; i < (int) select.size(); i++) {
            if(select[i] != entity) {
                if(select[i]->getTeam() != entity->getTeam()) {
                    select[i]->setAgressive(true);
                    select[i]->setFocus(entity);
                    select[i]->setFollow(entity);
                    select[i]->setGroupRadius(0);
                } else {
                    if(select[i]->getType() == UNIT_TYPE::WORKER) {
                        select[i]->setEntity(entity);
                        //select[i]->setGroupRadius(0);
                    }
                }
            }
        }
    }
}

/**
 * Insere uma unidade trabalhadora no edificio do tipo mina de ouro
 * @param building Ponteiro do edificio
 * @param unit Ponteiro da unidade
 * @return true se inseriu corretamente, false caso contrario
 */
bool Game::insertGoldSlot(Building* building, Unit* unit) {
    if((int) building->getSlot().size() < BuildingProperty::getInstance()->getMaxSlot(building->getType())) {
        unit->setPx(building->getPx());
        unit->setPy(building->getPy());
        unit->setGoldMine(building);
        building->insertSlot(unit);
        std::vector<Unit*>::iterator it;
        it = find (unitSet_.begin(), unitSet_.end(), unit);
        if(it != unitSet_.end()) {
            unitSet_.erase(it);
            return true;
        }
    }
    return false;
}

/**
 * Remove unidade trabalhadora de dentro da mina de ouro
 * @param building Ponteiro do edificio
 * @return true se removeu corretamente, false caso contrario
 */
bool Game::unloadGoldSlot(Building* building) {
    if(building->getSlot().size() > 0) {
        Unit* u = building->getSlot().back();
        //u->setPx(building->getPx());
        //u->setPy(building->getPy());
        u->setT(building->getPixelPath()[building->getPixelPath().size()-1].first, building->getPixelPath()[building->getPixelPath().size()-1].second);
        u->setPixelPath(building->getPixelPath());
        u->setGoldMine(NULL);
        unitSet_.push_back(u);
        building->releaseSlot();
        return true;
    }
    return false;
}

/**
 * Altera caracteristicas da unidade A para seguir uma entidade B
 * @param a Ponteiro da unidade A
 * @param b Ponteiro da entidade B
 */
void Game::aFollowb(Unit* a, Entity* b) {
    a->setFollow(b);
    a->setGroupRadius(0);
}

/**
 * Unidade unit para de seguir e faz busca de caminho para seu destino atual
 * @param unit Ponteiro da unidade
 */
void Game::stopFollow(Unit* unit) {
    unit->resetFollow();
    unitMove(unit, unit->getTx(), unit->getTy());
}


/**
 * Realiza um comando para um vetor de unidades
 * @param com String contendo o nome do comando
 * @param select Vetor de unidades
 * @return true se executou algum comando valido, false caso contrario
 */
bool Game::executeCommand(std::string com, std::vector<Unit*> select) {
    if(com.compare("Stop") == 0) {
        return selectStop(select);
    }
    return false;
}

/**
 * Realiza um comando para um edificio
 * @param com String contendo o nome do comando
 * @param select Ponteiro do edificio
 * @return true se executou algum comando valido, false caso contrario
 */
bool Game::executeCommand(std::string com, Building* select) {
    int unitType;
    if(com.compare("Unload") == 0) {
        return unloadGoldSlot(select);
    } else if((unitType = UnitProperty::getInstance()->getType(com)) != -1) {
        return insertBuildingQueue(select, unitType);
    }
    return false;
}

/**
 * Realiza um comando para um vetor de unidades a um ponto (px,py)
 * @param com String contendo o nome do comando
 * @param select Vetor de unidades
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @return true se executou algum comando valido, false caso contrario
 */
bool Game::executeCommand(std::string com, std::vector<Unit*> select, float px, float py) {
    int buildType;
    if(com.compare("Move") == 0) {
        return selectMove(select, px, py);
    } else if(com.compare("Attack") == 0) {
        return selectMoveAttack(select, px, py);
    } else if(com.compare("Gather") == 0) {
        return selectMoveGather(select, px, py);
    } else if((buildType = BuildingProperty::getInstance()->getType(com)) != -1) {
        return selectMoveBuild(select, px, py, buildType);
    }
    return false;
}

/**
 * Realiza um comando para um edificio a um ponto (px,py)
 * @param com String contendo o nome do comando
 * @param select Ponteiro do edificio
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @return true se executou algum comando valido, false caso contrario
 */
bool Game::executeCommand(std::string com, Building* select, float px, float py) {
    if(com.compare("Rally") == 0) {
        return buildingPath(select, px, py);
    }
    return false;
}
