#include "Map.h"

/**
 * Construtor
 * @param mapname Nome do arquivo contendo o mapa
 * @param configname Nome do arquivo contendo a configuracao do mapa
 */
Map::Map(const char* mapname, const char* configname) {
    int i, j;
    rows_ = 0;
    columns_ = 0;
    configname_ = configname;
    loadMap(mapname);
    tiles_ = new Tile**[rows_];
    for(i = 0; i < rows_; i++) {
        tiles_[i] = new Tile*[columns_];
    }
    for(i = 0; i < rows_; i++) {
        for(j = 0; j < columns_; j++) {
            tiles_[i][j] = new Tile();
        }
    }
    loadTiles();
}

/**
 * Destrutor
 */
Map::~Map() {
    printf("Deletando tiles de Map\n");
    int i, j;
    for(i = 0; i < rows_; i++) {
        for(j = 0; j < columns_; j++) {
            delete tiles_[i][j];
        }
    }
    for(i = 0; i < rows_; i++) {
        delete [] tiles_[i];
    }
    delete [] tiles_;
}

/**
 * @return Numero de linhas do mapa
 */
int Map::getRows() {
    return rows_;
}

/**
 * @return Numero de colunas do mapa
 */
int Map::getColumns() {
    return columns_;
}

/**
 * @return Nome do arquivo de configuracao do mapa
 */
const char* Map::getConfigName() {
    return configname_;
}

/**
 * @return Matriz do mapa
 */
std::vector<std::string> Map::getMapMatrix() {
    return mapMatrix_;
}

/**
 * @return Matriz de tiles do mapa
 */
Tile*** Map::getTiles() {
    return tiles_;
}

/**
 * Renderiza mapa
 */
void Map::render() {
    for(int i = 0; i < rows_; i++) {
        for(int j = 0; j < columns_; j++) {
            al_draw_bitmap(tiles_[i][j]->getBitmap(), i*TILE_WIDTH, j*TILE_HEIGHT, 0);
        }
    }
}

/**
 * Carrega mapa a partir de um arquivo de nome mapname
 * @param mapname
 */
void Map::loadMap(const char* mapname) {
    FILE* pFile = fopen(mapname, "r");
    char c;
    std::string temp;
	while(fscanf(pFile, "%c", &c) > 0) {
        if(c != '\n') {
            temp.push_back(c);
        }
		else {
            mapMatrix_.push_back(temp);
            temp.clear();
		}
    }
    mapMatrix_ = transpose(mapMatrix_);
	rows_ = mapMatrix_.size();
	columns_ = mapMatrix_[0].size();
	fclose(pFile);
}

/**
 * Carrega o tipo de tile nos tiles do mapa
 */
void Map::loadTiles() {
    int i, j;
    for(i = 0; i < rows_; i++) {
        for(j = 0; j < columns_; j++) {
            tiles_[i][j]->setType(TileProperty::getInstance()->getType(mapMatrix_[i][j]));
        }
    }
}

// Serve apenas para compatibilidade do sistema de coordenadas do Allegro
/**
 * Faz a transposta da matriz matrix
 * @param matrix Matriz contendo mapa
 * @return Transposta da matriz matrix
 */
std::vector<std::string> Map::transpose(std::vector<std::string> matrix) {
    std::vector<std::string> transp;
    std::string aux;
    for(int i = 0; i < (int) matrix[0].size(); i++) {
        for(int j = 0; j < (int) matrix.size(); j++) {
            aux.push_back(matrix[j][i]);
        }
        transp.push_back(aux);
        aux.clear();
    }

    return transp;
}
