#include "ResultState.h"
#include "PlayingState.h"

ResultState* ResultState::instance = NULL;

/**
 * Construtor
 * @param display Tela do jogo
 * @param timer Tempo de atualizacao
 * @param queue Fila de eventos
 */
ResultState::ResultState(ALLEGRO_DISPLAY* display, ALLEGRO_TIMER* timer, ALLEGRO_EVENT_QUEUE* queue) :
    State(display, timer, queue) {
    redraw = false;
    font = al_load_ttf_font("fonts/times.ttf",20,0);
}

/**
 * Destrutor
 */
ResultState::~ResultState() {
}

/**
 * Le entrada
 * @param sm Gerenciador de estados
 */
void ResultState::readInput(StateManager* sm) {
    ALLEGRO_EVENT event;
    al_wait_for_event(queue_, &event);
    if (event.type == ALLEGRO_EVENT_KEY_DOWN) {
        if (event.keyboard.keycode == ALLEGRO_KEY_ESCAPE) {
            sm->quit();
        }
        else if (event.keyboard.keycode == ALLEGRO_KEY_ENTER) {
            //stateManager->changeState(State::idToState[STATE::PLAYING]);
            //printf("APERTEI ENTER NO PAUSE\n");
            //delete PlayingState::getInstance(display_,timer_,queue_)->getGame();
            PlayingState::getInstance(display_,timer_,queue_)->restartGame();
            sm->changeState(PlayingState::getInstance(display_,timer_,queue_));
        }
    }
    if (event.type == ALLEGRO_EVENT_TIMER) {
        redraw = true;
    }
    if (event.type == ALLEGRO_EVENT_DISPLAY_RESIZE) {
        al_acknowledge_resize(display_);
        redraw = true;
    }
}

/**
 * Atualiza
 * @param sm Gerenciador de estados
 */
void ResultState::update(StateManager* sm) {

}

/**
 * Renderiza
 * @param sm Gerenciador de estados
 */
void ResultState::render(StateManager* sm) {
    if(redraw && al_is_event_queue_empty(queue_)) {
        redraw = false;
        al_clear_to_color(al_map_rgb(0,0,0));
        al_draw_text(font, al_map_rgb(255,255,255), 0, 50, ALLEGRO_ALIGN_LEFT, "You are on result state");
        al_draw_textf(font, winner_->getColor(), 0, 100, ALLEGRO_ALIGN_LEFT, "Team %d won the game", winner_->getTeam());
        al_draw_text(font, al_map_rgb(255,255,255), 0, 150, ALLEGRO_ALIGN_LEFT, "PRESS ENTER to restart the game");
        al_draw_text(font, al_map_rgb(255,255,255), 0, 200, ALLEGRO_ALIGN_LEFT, "PRESS ESC to quit");
        al_flip_display();
    }
}

/**
 * Atribui o jogador vencedor
 * @param winner Ponteiro para o jogador vencedor
 */
void ResultState::setWinner(Player* winner) {
    winner_ = winner;
}

