#include "Mymath.h"

namespace mymath {
/**
 * Devolve a distancia euclidiana entre os pontos (x1,y1) e (x2,y2)
 * @param x1 Coordenada x do ponto 1
 * @param y1 Coordenada y do ponto 1
 * @param x2 Coordenada x do ponto 1
 * @param y2 Coordenada y do ponto 2
 * @return Distancia euclidiana entre (x1,y1) e (x2,y2)
 */
float distance(float x1, float y1, float x2, float y2) {
    return sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
}

/**
 * Verifica se dois circulos estao em colisao
 * @param x1 Coordenada x do circulo 1
 * @param y1 Coordenada y do circulo 1
 * @param r1 Raio do circulo 1
 * @param x2 Coordenada x do circulo 2
 * @param y2 Coordenada y do circulo 2
 * @param r2 Raio do circulo 2
 * @return true se o circulo 1 esta dentro do circulo 2, false caso contrario
 */
bool collide(float x1, float y1, float r1, float x2, float y2, float r2) {
    /* O metodo de empurrar sempre empurra de tal forma que a distancia entre os dois circulos seja r1+r2,
        Entao coloquei o -5 não ficar detectando colisao sempre que estiverem com essa distancia */
    return (powDistance(x1,y1,x2,y2) < (r1+r2)*(r1+r2)-5);
}

/**
 * Devolve o quadrado da distancia euclidiana entre os pontos (x1,y1) e (x2,y2)
 * @param x1 Coordenada x do ponto 1
 * @param y1 Coordenada y do ponto 1
 * @param x2 Coordenada x do ponto 1
 * @param y2 Coordenada y do ponto 2
 * @return Quadrado da distancia euclidiana entre (x1,y1) e (x2,y2)
 */
float powDistance(float x1, float y1, float x2, float y2) {
    return (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2);
}

/**
 * Verifica se dois circulos estao em colisao usando powDistance
 * @param x1 Coordenada x do ponto 1
 * @param y1 Coordenada y do ponto 1
 * @param x2 Coordenada x do ponto 1
 * @param y2 Coordenada y do ponto 2
 * @return true se o circulo 1 esta dentro do circulo 2, false caso contrario
 */
bool inrange(float x1, float y1, float r1, float x2, float y2, float r2) {
    return (powDistance(x1,y1,x2,y2) <= (r1+r2)*(r1+r2));
}

/**
 * Verifica se um circulo esta em colisao com um retangulo
 * @param x Coordenada x do circulo
 * @param y Coordenada y do circulo
 * @param r Raio do circulo
 * @param rectx1 Primeira coordenada x do retangulo
 * @param recty1 Primeira coordenada y do retangulo
 * @param rectx2 Segunda coordenada x do retangulo
 * @param recty2 Segunda coordenada y do retangulo
 * @return true se o circulo esta em colisao com o retangulo
 */
bool inside(float x, float y, float r, float rectx1, float recty1, float rectx2, float recty2) {
    float xmin,ymin,xmax,ymax;
    float xnear, ynear; /* Escolhemos o ponto mais proximo do centro do circulo */
    xmin = rectx2;
    ymin = recty2;
    xmax = rectx1;
    ymax = recty1;
    if(rectx1 < rectx2) {
        xmin = rectx1;
        xmax = rectx2;
    }
    if(recty1 < recty2) {
        ymin = recty1;
        ymax = recty2;
    }
    if(x >= xmin && x <= xmax) {
        if(y >= ymin && y <= ymax) {
            return true;
        }
    }
    if(x < xmin) xnear = xmin;
    else if(x > xmax) xnear = xmax;
    else xnear = x;
    if(y < ymin) ynear = ymin;
    else if(y > ymax) ynear = ymax;
    else ynear = y;
    if(distance(x,y,xnear,ynear) <= r) return true;
    return false;
}

/**
 * @return Inteiro x/d
 */
int div(float x, int d) {
    int r = x/d;
    return r;
}
}
