#include "Node.h"

/**
 * Construtor
 * @param x Coordenada x do no'
 * @param y Coordenada y do no'
 */
Node::Node(int x, int y) : x_(x), y_(y) {
    type_ = -1;
    tileType_ = -1;
    parent_ = NULL;
    estimate_ = -1;
    cost_ = -1;
    dist_ = -1;
    walkable_ = true;
    onPath_ = false;
    choke_ = -1;
    zone_ = -1;
    team_ = -1;
}

/**
 * Destrutor
 */
Node::~Node() {

}

/**
 * @return Coordenada x
 */
int Node::getX() {
    return x_;
}

/**
 * Atribui a coordenada x do no'
 * @param x Coordenada x
 */
void Node::setX(int x) {
    x_ = x;
}

/**
 * @return Coordenada y
 */
int Node::getY() {
    return y_;
}

/**
 * Atribui a coordenada y do no'
 * @param y Coordenada y
 */
void Node::setY(int y) {
    y_ = y;
}

/**
 * @return Tipo do no'
 */
int Node::getType() {
    return type_;
}

/**
 * Atribui o tipo do no'
 * @param type Tipo do no'
 */
void Node::setType(int type) {
    type_ = type;
}

/**
 * @return Tipo do tile do no'
 */
int Node::getTileType() {
    return tileType_;
}

/**
 * Atribui o tipo do tile do no'
 * @param tileType Tipo do tile do no'
 */
void Node::setTileType(int tileType) {
    tileType_ = tileType;
}

/**
 * @return Zona do no'
 */
int Node::getZone() {
    return zone_;
}

/**
 * Atribui zona ao no'
 * @param zone Valor da zona
 */
void Node::setZone(int zone) {
    zone_ = zone;
}

/**
 * @return Time do edificio que esta no no'
 */
int Node::getTeam() {
    return team_;
}

/**
 * Atribui time ao no' que possui algum edificio
 */
void Node::setTeam(int team) {
    team_ = team;
}

/**
 * @return Custo da no'
 */
float Node::getCost() {
    return cost_;
}

/**
 * Atribui um custo ao no'
 * @param cost Custo do no'
 */
void Node::setCost(float cost) {
    cost_ = cost;
}

/**
 * @return Distancia do no'
 */
float Node::getDist() {
    return dist_;
}

/**
 * Atribui distancia ao no'
 * @param dist Distancia do no'
 */
void Node::setDist(float dist) {
    dist_ = dist;
}

/**
 * @return Distancia estimada do no'
 */
float Node::getEstimate() {
    return estimate_;
}

/**
 * Atribui distancia estimada do no'
 * @param estimate Distancia estimada do no'
 */
void Node::setEstimate(float estimate) {
    estimate_ = estimate;
}

/**
 * @return true se o no' for andavel, false caso contrario
 */
bool Node::isWalkable() {
    return walkable_;
}

/**
 * Atribui a caractistica walkable
 * @param walkable Valor booleano
 */
void Node::setWalkable(bool walkable) {
    walkable_ = walkable;
}

/**
 * @return true se esse no' estiver no caminho da unidade que realizou a busca de caminho, false caso contrario
 */
bool Node::isOnPath() {
    return onPath_;
}

/**
 * Atribui onPath
 * @param onPath
 */
void Node::setOnPath(bool onPath) {
    onPath_ = onPath;
}

/**
 * @return -1 se nao for ponto de estrangulamento, N inteiro maior ou igual a 0 se for ponto de estrangulamento mais proximo do time N
 */
int Node::getChoke() {
    return choke_;
}

/**
 * Atribui ponto de estrangulamento
 * @param choke
 */
void Node::setChoke(int choke) {
    choke_ = choke;
}

/**
 * @return Pai do no'
 */
Node* Node::getParent() {
    return parent_;
}

/**
 * Atribui pai ao no'
 * @param parent Pai do no'
 */
void Node::setParent(Node* parent) {
    parent_ = parent;
}
