#include "Gui.h"

/**
 * Construtor
 * @param theme Tema da GUI
 * @param display Tela do jogo
 */
Gui::Gui(const Theme & theme, ALLEGRO_DISPLAY* display) {
    d = new Dialog(theme,display,30,40);
    button = new Button*[9];
    for(int i = 0; i < 9; i++) {
        button[i] = new Button("");
    }
    for(int i = 0; i < 3; i++) {
        for(int j = 0; j < 3; j++) {
            d->add(*button[i*3+j], 24+2*j, 31+3*i, 2, 3);
        }
    }
    d->prepare();
    selectBuilding_ = NULL;
}

/**
 * Destrutor
 */
Gui::~Gui() {
    printf("Deletando button e dialog de Gui\n");
    for(int i = 0; i < 9; i++) {
        delete button[i];
    }
    delete [] button;
    delete d;
}

/**
 * @return Coordenada x em pixel da GUI
 */
float Gui::getPx() {
    return px_;
}

/**
 * Atribui valor da coordenada x da GUI
 * @param px Coordenada x em pixel da GUI
 */
void Gui::setPx(float px) {
    px_ = px;
}

/**
 * @return Coordenada y em pixel da GUI
 */
float Gui::getPy() {
    return py_;
}

/**
 * Atribui valor da coordenada y da GUI
 * @param py Coordenada y em pixel da GUI
 */
void Gui::setPy(float py) {
    py_ = py;
}

/**
 * @return Largura da GUI
 */
float Gui::getWidth() {
    return width_;
}

/**
 * Atribui valor da largura da GUI
 * @param width Largura da GUI
 */
void Gui::setWidth(float width) {
    width_ = width;
}

/**
 * @return Altura da GUI
 */
float Gui::getHeight() {
    return height_;
}

/**
 * Atribui valor da altura da GUI
 * @param height Altura da GUI
 */
void Gui::setHeight(float height) {
    height_ = height;
}

/**
 * @return String com o nome da acao que foi pressionada
 */
std::string Gui::getAction() {
    d->run_step(false);
    for(int i = 0; i < 9; i++) {
        if(button[i]->get_pushed()) {
            return button[i]->get_text();
        }
    }
    return "";
}

/**
 * Altera as dimensoes da GUI
 * @param px Coordenada x em pixel da GUI
 * @param py Coordenada y em pixel da GUI
 * @param width Largura da GUI
 * @param height Altura da GUI
 */
void Gui::setRectangle(float px, float py, float width, float height) {
    px_ = px;
    py_ = py;
    width_ = width;
    height_ = height;
}

/**
 * Atribui um game a GUI
 * @param game Ponteiro do game
 */
void Gui::setGame(Game* game) {
    game_ = game;
}

/**
 * Atrubui um vetor de unidades selecionada
 */
void Gui::setSelect(std::vector<Unit*> select) {
    select_ = select;
}

/**
 * Atrubui um edificio selecionado
 */
void Gui::setSelectBuilding(Building* selectBuilding) {
    selectBuilding_ = selectBuilding;
}

/**
 * Atualiza GUI de acordo com o estado do simulador
 * @param aState Estado do simulador
 */
void Gui::update(int aState) {
    aState_ = aState;
    if(aState == ACTION_STATE::SELECT_UNIT) {
        Unit* u = select_[0];
        for(int i = 0; i < (int) u->getAction().size(); i++) {
            std::string s = u->getAction()[i];
            button[i]->set_text(s);
        }
    } else if(aState == ACTION_STATE::SELECT_BUILDING) {
        Building* b = selectBuilding_;
        if(b->isActive()) {
            for(int i = 0; i < (int) b->getAction().size(); i++) {
                std::string s = b->getAction()[i];
                button[i]->set_text(s);
            }
        }
    } else {
        for(int i = 0; i < 9; i++) button[i]->set_text("");
    }
}

/**
 * Renderiza GUI
 */
void Gui::render() {
    al_draw_filled_rectangle(px_,py_,px_+width_,py_+height_,d->get_theme().bg);
    d->run_step(false);
    //d->draw();
    if(aState_ == ACTION_STATE::SELECT_UNIT) {
        Unit* u = select_[0];
        for(int i = 0; i < (int) u->getAction().size(); i++) {
            button[i]->draw();
        }
    } else if(aState_ == ACTION_STATE::SELECT_BUILDING) {
        Building* b = selectBuilding_;
        if(b->isActive()) {
            for(int i = 0; i < (int) b->getAction().size(); i++) {
                button[i]->draw();
            }
        }
    }
    renderUnit();
    renderBuilding();
    renderStatus();
}

/**
 * Renderiza GUI para unidades
 */
void Gui::renderUnit() {
    if(select_.size() == 1) {
        Unit* u = select_[0];
        float initialX = px_;
        float initialY = py_;
        int num = 0;
        int incr = 20;
        al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX, initialY+incr*(num++), ALLEGRO_ALIGN_LEFT, "%s", u->getName().c_str());
        al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX, initialY+incr*(num++), ALLEGRO_ALIGN_LEFT, "HP: %.0f/%.0f", u->getHp(), u->getMaxHp());
        al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX, initialY+incr*(num++), ALLEGRO_ALIGN_LEFT, "Damage: %.0f", u->getDamage());
        al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX, initialY+incr*(num++), ALLEGRO_ALIGN_LEFT, "A.Speed: %.2f/%.2f", u->getCooldown(),u->getAttackSpeed());
        al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX, initialY+incr*(num++), ALLEGRO_ALIGN_LEFT, "Range: %.1f", u->getRange());
    }
    else if(select_.size() > 0) {
        for(int i = 0; i < (int) select_.size(); i++) {
            ALLEGRO_COLOR color;
            float percentHP = select_[i]->getHp()/select_[i]->getMaxHp();
            if(percentHP >= 0.5) {
                color = al_map_rgb(255*(1-percentHP)*2,255,0);
            } else {
                color = al_map_rgb(255,255*(percentHP-1)*2,0);
            }
            ALLEGRO_BITMAP* icon = NULL;
            icon = UnitProperty::getInstance()->getIcon(select_[i]->getType());
            al_draw_filled_rectangle(px_+24*(i%20),py_+24*(i/20),px_+24*(i%20)+24,py_+24*(i/20)+24,color);
            al_draw_scaled_bitmap(icon, 0,0,64,64,px_+24*(i%20),py_+24*(i/20),24,24,0);
        }
    }
}

/**
 * Renderiza GUI para edificios
 */
void Gui::renderBuilding() {
    if(selectBuilding_ != NULL) {
        float initialX = px_;
        float initialY = py_;
        int num = 0;
        int incr = 20;
        Building* b = selectBuilding_;
        al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX, initialY+incr*(num++), ALLEGRO_ALIGN_LEFT, "%s", b->getName().c_str());
        al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX, initialY+incr*(num++), ALLEGRO_ALIGN_LEFT, "HP: %.0f/%.0f", b->getHp(), b->getMaxHp());
        if(!b->isActive()) {
            float percent = b->getBuildingTime()/b->getTime();
            al_draw_line(initialX+125,initialY+15,initialX+225,initialY+15,al_map_rgb(0,0,0), 6);
            al_draw_line(initialX+125,initialY+15,initialX+125+100*percent,initialY+15,al_map_rgb(0,255,0), 5);
            al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX+235, initialY, ALLEGRO_ALIGN_LEFT, "%.1f/%.1f", b->getBuildingTime(),b->getTime());
        }
        else if(b->getQueue().size() > 0) {
            for(int i = 0; i < (int) b->getQueue().size(); i++) {
                ALLEGRO_BITMAP* icon = NULL;
                icon = UnitProperty::getInstance()->getIcon(b->getQueue()[i]);
                al_draw_scaled_bitmap(icon, 0,0,64,64,initialX+100,initialY+24*i,24,24,0);
            }
            float percent = b->getTrainingTime()/UnitProperty::getInstance()->getTime(b->getQueue()[0]);
            // Fundo cinza
            al_draw_line(initialX+125,initialY+15,initialX+225,initialY+15,al_map_rgb(0,0,0), 6);
            // Barra tempo
            al_draw_line(initialX+125,initialY+15,initialX+125+100*percent,initialY+15,al_map_rgb(0,255,0), 5);
            al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX+235, initialY, ALLEGRO_ALIGN_LEFT, "%.1f/%.1f", b->getTrainingTime(),UnitProperty::getInstance()->getTime(b->getQueue()[0]));
        }
        else if(b->getType() == BuildingProperty::getInstance()->getType("Gold_Mine")) {
            //ALLEGRO_BITMAP* icon = NULL;
            //icon = UnitProperty::getInstance()->getIcon(b->getSlot()[i]->getType());
            al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX+100, initialY, ALLEGRO_ALIGN_LEFT, "%d/%d Workers", b->getSlot().size(), b->getMaxSlot());
            float percent = b->getMiningTime()/b->getMiningCooldown();
            // Fundo cinza
            al_draw_line(initialX+100,initialY+30,initialX+200,initialY+30,al_map_rgb(0,0,0), 6);
            // Barra tempo
            al_draw_line(initialX+100,initialY+30,initialX+100+100*percent,initialY+30,al_map_rgb(0,255,0), 5);
            al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX+210, initialY+20, ALLEGRO_ALIGN_LEFT, "%.1f/%.1f", b->getMiningTime(), b->getMiningCooldown());
            //al_draw_scaled_bitmap(icon, 0,0,64,64,initialX+100,initialY,24,24,0);
        }

    }
}

/**
 * Renderiza GUI para estado dos jogadores
 */
void Gui::renderStatus() {
    int initialX = 600;
    int initialY = 0;
    std::vector<Player*> player = game_->getPlayer();
    al_draw_filled_rectangle(initialX,initialY,800,40,al_map_rgb(255,255,255));
        for(int i = 0; i < (int) player.size(); i++) {
            al_draw_filled_rectangle(initialX, initialY+i*20, initialX+20, initialY+20+i*20, al_map_rgb(255*i,0,255*(1-i)));
            al_draw_textf(d->get_theme().font, d->get_theme().fg, initialX+25, initialY+i*20, ALLEGRO_ALIGN_LEFT,("G: %d B: %d U: %d/%d"),
                          player[i]->getGold(),player[i]->getBuildingSet().size(),player[i]->getUnitCount(), player[i]->getUnitCap());
    }
}



