#include "Building.h"

/**
 * Construtor
 * @param x Coordenada x
 * @param y Coordenada y
 * @param type Tipo do edificio
 * @param team Time do edificio
 */
Building::Building(int x, int y, int type, int team) : x_(x), y_(y) {
    type_ = type;
    team_ = team;
    px_ = (x*TILE_WIDTH+(x+getDimension())*TILE_WIDTH)/2;
    py_ = (y*TILE_HEIGHT+(y+getDimension())*TILE_HEIGHT)/2;
    hp_ = getMaxHp()*0.1;
    trainingTime_ = 0;
    buildingTime_ = 0;
    miningTime_ = 0;
    constructing_ = false;
}

/**
 * Destrutor
 */
Building::~Building() {
}

/**
 * Altera os atributos do edificio para edificio disponivel
 */
void Building::setReady() {
    hp_ = BuildingProperty::getInstance()->getMaxHp(type_);
    buildingTime_ = BuildingProperty::getInstance()->getTime(type_);
}

/**
 * @return Coordenada x
 */
int Building::getX() {
    return x_;
}

/**
 * Atribui a coordenada x do edificio
 * @param x Coordenada x
 */
void Building::setX(int x) {
    x_ = x;
}

/**
 * @return Coordenada y
 */
int Building::getY() {
    return y_;
}

/**
 * Atribui a coordenada y do edificio
 * @param y Coordenada y
 */
void Building::setY(int y) {
    y_ = y;
}

/**
 * @return Dimensao do edificio
 */
int Building::getDimension() {
    return BuildingProperty::getInstance()->getDimension(type_);
}

/**
 * @return Bitmap do edificio
 */
ALLEGRO_BITMAP* Building::getBitmap() {
    return BuildingProperty::getInstance()->getBitmap(type_);
}

/**
 * @return Vetor de acoes do edificio
 */
std::vector<std::string> Building::getAction() {
    return BuildingProperty::getInstance()->getAction(type_);
}

/**
 * @return Vida maxima do edificio
 */
float Building::getMaxHp() {
    return BuildingProperty::getInstance()->getMaxHp(type_);
}

/**
 * @return Raio do edificio
 */
float Building::getRadius() {
    return BuildingProperty::getInstance()->getRadius(type_);
}

/**
 * @return Nome do edificio
 */
std::string Building::getName() {
    return BuildingProperty::getInstance()->getName(type_);
}

/**
 * @return Preco do edificio
 */
int Building::getCost() {
    return BuildingProperty::getInstance()->getCost(type_);
}

/**
 * @return Tempo de construcao do edificio
 */
float Building::getTime() {
    return BuildingProperty::getInstance()->getTime(type_);
}

/**
 * @return Numero maximo de slots do edificio
 */
int Building::getMaxSlot() {
    return BuildingProperty::getInstance()->getMaxSlot(type_);
}

/**
 * @return Numero maximo de producao de unidades do edificio
 */
int Building::getMaxQueue() {
    return BuildingProperty::getInstance()->getMaxQueue(type_);
}

/**
 * @return Quantidade de ouro ganho que por tick que o edificio gera
 */
int Building::getMiningGain() {
    return BuildingProperty::getInstance()->getMiningGain(type_);
}

/**
 * @return Tempo de recarga de cada tick de incremento de ouro
 */
float Building::getMiningCooldown() {
    return BuildingProperty::getInstance()->getMiningCooldown(type_);
}

/**
 * @return true se for do tipo edificio, false caso contrario
 */
bool Building::isBuilding() {
    return true;
}

/**
 * @return Vetor dos tipos de unidades sendo produzidas no edificio
 */
std::vector<int> Building::getQueue() {
    return queue_;
}

/**
 * @return Tempo de producao da unidade atual
 */
float Building::getTrainingTime() {
    return trainingTime_;
}

/**
 * @return Tempo de construcao do edificio
 */
float Building::getBuildingTime() {
    return buildingTime_;
}

/**
 * @return Tempo de coleta de recurso
 */
float Building::getMiningTime() {
    return miningTime_;
}

/**
 * Atribui o tempo de construcao do edificio
 * @param buildingTime Tempo de construcao do edificio
 */
void Building::setBuildingTime(float buildingTime) {
    buildingTime_ = buildingTime;
}

/**
 * Atribui o tempo de treinamento da unidade sendo produzida no edificio
 * @param trainingTime Tempo de treinamento do edificio
 */
void Building::setTrainingTime(float trainingTime) {
    trainingTime_ = trainingTime;
}

/**
 * Atribui o tempo de coleta de recurso no edificio
 * @param miningTime
 */
void Building::setMiningTime(float miningTime) {
    miningTime_ = miningTime;
}

/**
 * @return true se o edificio esta sendo construido, false caso contrario
 */
bool Building::isConstructing() {
    return constructing_;
}

/**
 * Atribui um valor booleano a variavel constructing
 * @param constructing Valor que define se esta sendo construido
 */
void Building::setConstructing(bool constructing) {
    constructing_ = constructing;
}

/**
 * Atribui vetor de producao de unidades
 * @param queue Vetor de producao de unidades
 */
void Building::setQueue(std::vector<int> queue) {
    queue_ = queue;
}

/**
 * Insere um tipo de unidade a ser produzida no edificio
 * @param unitType Tipo da unidade
 */
bool Building::insertQueue(int unitType) {
    queue_.push_back(unitType);
    return true;
}

/**
 * Remove um tipo de unidade na fila de producao
 * @param pos Posicao do tipo da unidade a ser removida
 */
bool Building::removeQueue(int pos) {
    if((int) queue_.size() > pos) {
        if(pos == 0) trainingTime_ = 0;
        queue_.erase(queue_.begin() + pos);
        return true;
    }
    return false;
}

/**
 * @return true se a construcao esta ativa, false caso contrario
 */
bool Building::isActive() {
    return (buildingTime_ >= BuildingProperty::getInstance()->getTime(type_));
}

/**
 * Constroi edificio
 */
void Building::construct() {
    constructing_ = true;
}

/**
 * Insere uma unidade dentro do edificio
 * @param unit Ponteiro da unidade
 */
bool Building::insertSlot(Unit* unit) {
    slot_.push_back(unit);
    return true;
}

/**
 * Libera uma unidade de dentro do edificio
 */
bool Building::releaseSlot() {
    slot_.pop_back();
    return true;
}

/**
 * @return Vetor de unidades dentro do edificio
 */
std::vector<Unit*> Building::getSlot() {
    return slot_;
}

/**
 * Atualiza edificio
 */
void Building::update() {
    if(constructing_) {
        buildingTime_ = buildingTime_ + 1.0/(MAX_FPS);
        float percent = 1.0/(MAX_FPS)/getTime();
        hp_ = hp_ + (getMaxHp()*0.9)*percent;
        if(buildingTime_ > getTime()) {
            if(hp_ > getMaxHp()) hp_ = getMaxHp();
            buildingTime_ = getTime();
        }
        constructing_ = false;
    }
    if(queue_.size() > 0) {
        trainingTime_ = trainingTime_ + 1.0/(MAX_FPS);
        //printf("training time %f\n", trainingTime_);
    }
    if(slot_.size() > 0) {
        miningTime_ = miningTime_ + slot_.size()*(1.0/(MAX_FPS));
    }
}

/**
 * Renderiza edificio
 */
void Building::render() {
    ALLEGRO_COLOR color;
    float percent = buildingTime_/getTime();
    float percentHP = hp_/getMaxHp();
    if(percentHP >= 0.5) {
        color = al_map_rgb(255*(1.0-percentHP)*2,255,0);
    } else {
        color = al_map_rgb(255,255*(percentHP)*2,0);
    }
    // Sprite
    al_draw_tinted_bitmap(getBitmap(), al_map_rgba(255, 255, 255, 255*percent), px_-(getDimension()*TILE_WIDTH)/2, py_-(getDimension()*TILE_HEIGHT)/2, 0);
    // Fundo cinza da barra de HP
    al_draw_line(px_-getRadius(),py_-getRadius(),px_+getRadius(),py_-getRadius(),al_map_rgba(0,0,0,128), 4);
    // Barra de HP
    al_draw_line(px_-getRadius(),py_-getRadius(),px_+getRadius()-(2*getRadius())*(1-percentHP),py_-getRadius(),color, 3);
}
