#include "Projectile.h"

/**
 * Construtor
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @param damage Dano do projetil
 * @param type Tipo do projetil
 * @param target Alvo do projetil
 */
Projectile::Projectile(float px, float py, float damage, int type, Entity* target) : px_(px), py_(py), damage_(damage), type_(type), target_(target) {
    angle_ = 0;
}

/**
 * Destrutor
 */
Projectile::~Projectile() {

}

/**
 * @return Coordenada x em pixel do projetil
 */
float Projectile::getPx() {
    return px_;
}

/**
 * @return Coordenada y em pixel do projetil
 */
float Projectile::getPy() {
    return py_;
}

/**
 * @return Angulo do projetil
 */
float Projectile::getAngle() {
    return angle_;
}

/**
 * @return Dano do projetil
 */
float Projectile::getDamage() {
    return damage_;
}

/**
 * @return Tipo do projetil
 */
int Projectile::getType() {
    return type_;
}

/**
 * @return Alvo do projetil
 */
Entity* Projectile::getTarget() {
    return target_;
}

/**
 * @return Velocidade do projetil
 */
float Projectile::getSpeed() {
    return ProjectileProperty::getInstance()->getSpeed(type_);
}

/**
 * @return Raio do projetil
 */
float Projectile::getRadius() {
    return ProjectileProperty::getInstance()->getRadius(type_);
}

/**
 * @return Bitmap do projetil
 */
ALLEGRO_BITMAP* Projectile::getBitmap() {
    return ProjectileProperty::getInstance()->getBitmap(type_);
}

/**
 * @return Vetor do proximo movimento do projetil
 */
std::pair<float,float> Projectile::getNextMovement() {
    float tpx, tpy;
    float a,b;
    if(target_->getHp() > 0) {
        tpx = target_->getPx();
        tpy = target_->getPy();
        a = tpx-px_;
        b = tpy-py_;
        return std::make_pair(a/sqrt(a*a+b*b), b/sqrt(a*a+b*b));
    }
    return std::make_pair(0,0);
}

/**
 * Movimenta projetil usando um vetor d de direcao
 * @param d Vetor de movimentacao
 */
void Projectile::move(std::pair<float,float> d) {
    px_ = px_ + d.first*getSpeed()/MAX_FPS;
    py_ = py_ + d.second*getSpeed()/MAX_FPS;

    if(d.first != 0 || d.second != 0) {
        angle_ = acos(d.first/sqrt(d.first*d.first+d.second*d.second));
        if(d.second < 0) angle_ = -angle_;
    }

}

/**
 * Atualiza projetil
 */
void Projectile::update() {
    move(getNextMovement());
}

/**
 * Renderiza
 */
void Projectile::render() {
    al_draw_rotated_bitmap(getBitmap(), getRadius(), getRadius(), px_, py_, angle_, 0);
}
