#include "Unit.h"

/**
 * Construtor
 * @param px Coordenada x em pixel
 * @param py Coordenada y em pixel
 * @param type Tipo da unidade
 * @param team Time da unidade
 */
Unit::Unit(float px, float py, int type, int team) {
    px_ = px;
    py_ = py;
    type_ = type;
    team_ = team;
    hp_ = UnitProperty::getInstance()->getMaxHp(type);
    tx_ = px_;
    ty_ = py_;
    lastMove_ = std::make_pair(0,0);
    groupRadius_ = 0;
    cooldown_ = 0;
    agressive_ = false;
    hold_ = false;
    wait_ = false;
    pushed_ = false;
    target_ = NULL;
    focus_ = NULL;
    follow_ = NULL;
    signal_ = 0;
    build_ = -1;
    movingTime_ = 0;
    actionTime_ = 0;
    renderAct_ = false;
    angle_ = 0;
    xfollow = -1;
    yfollow = -1;
    repath_ = false;
    entity_ = NULL;
    collide = NULL;
    dist = -1;
    ncollide = 0;
    goldMine_ = NULL;
    movePosition = std::make_pair(-1,-1);
}

/**
 * Destrutor
 */
Unit::~Unit() {
}

/**
 * @return Coordenada x de onde a unidade quer se mover
 */
float Unit::getTx() {
    return tx_;
}

/**
 * @return Coordenada y de onde a unidade quer se mover
 */
float Unit::getTy() {
    return ty_;
}

/**
 * @return Par de coordenadas de onde a unidade quer se mover
 */
std::pair<float,float> Unit::getT() {
    return std::make_pair(tx_, ty_);
}

/**
 * @return Sinal de rotacao, usado pra colisao do tipo 2
 */
int Unit::getSignal() {
    return signal_;
}

/**
 * @return true se a unidade nao pode ser empurrada, false caso contrario
 */
bool Unit::isHold() {
    return hold_;
}

/**
 * @return true se a unidade esta no modo agressivo, false caso contrario
 */
bool Unit::isAgressive() {
    return agressive_;
}

/**
 * @return Valor do tempo de recarga de tiro da unidade
 */
float Unit::getCooldown() {
    return cooldown_;
}

/**
 * @return Raio do grupo
 */
float Unit::getGroupRadius() {
    return groupRadius_;
}

/**
 * @return Ponteiro do alvo da unidade
 */
Entity* Unit::getTarget() {
    return target_;
}

/**
 * @return Ponteiro do foco da unidade
 */
Entity* Unit::getFocus() {
    return focus_;
}

/**
 * @return Ponteiro da unidade que esta sendo seguida
 */
Entity* Unit::getFollow() {
    return follow_;
}

/**
 * @return Vetor da ultima movimentacao da unidade
 */
std::pair<float,float> Unit::getLastMove() {
    return lastMove_;
}

/**
 * @return Caminho de colisao do tipo 2
 */
std::vector<std::pair<float,float>> Unit::getCollisionPath() {
    return collisionPath_;
}

/**
 * @return Tipo do edificio que a unidade quer construir
 */
int Unit::getBuild() {
    return build_;
}

/**
 * @return Ponteiro do edificio gold mine em que a unidade esta
 */
Entity* Unit::getGoldMine() {
    return goldMine_;
}

///////////// Sets

/**
 * Atribui valor da coordenada x de onde a unidade quer se mover
 * @param tx Coordenada x
 */
void Unit::setTx(float tx) {
    tx_ = tx;
}

/**
 * Atribui valor da coordenada y de onde a unidade quer se mover
 * @param ty Coordenada y
 */
void Unit::setTy(float ty) {
    ty_ = ty;
}

/**
 * Atribui as coordenadas de onde a unidade quer se movimenta
 * @param tx Coordenada x
 * @param ty Coordenada y
 */
void Unit::setT(float tx, float ty) {
    tx_ = tx;
    ty_ = ty;
}

/**
 * Atribui de signal da unidade
 * @param signal Sinal de rotacao
 */
void Unit::setSignal(int signal) {
    signal_ = signal;
}

/**
 * Atribui de hold da unidade
 * @param hold Valor booleano de hold
 */
void Unit::setHold(bool hold) {
    hold_ = hold;
}

/**
 * Atribui agressividade da unidade
 * @param agressive Valor booleano de agressividade da unidade
 */
void Unit::setAgressive(bool agressive) {
    agressive_ = agressive;
}

/**
 * Atribui valor de tempo de recarga de ataque da unidade
 * @param cooldown Tempo de recarga de ataque da unidade
 */
void Unit::setCooldown(float cooldown) {
    cooldown_ = cooldown;
}

/**
 * Atribui valor do raio do grupo da unidade
 * @param groupRadius Raio do grupo
 */
void Unit::setGroupRadius(float groupRadius) {
    groupRadius_ = groupRadius;
}

/**
 * Atribui entidade alvo da unidade
 * @param target Ponteiro alvo da unidade
 */
void Unit::setTarget(Entity* target) {
    target_ = target;
}

/**
 * Atribui entidade foco da unidade
 * @param focus Ponteiro foco da unidade
 */
void Unit::setFocus(Entity* focus) {
    focus_ = focus;
}

/**
 * Atribui entidade follow da unidade
 * @param follow Ponteiro follow da unidade
 */
void Unit::setFollow(Entity* follow) {
    follow_ = follow;
}

/**
 * Atribui vetor de ultima movimentacao da unidade
 * @param lastMove Vetor de ultima movimentacao da unidade
 */
void Unit::setLastMove(std::pair<float,float> lastMove) {
    lastMove_ = lastMove;
}

/**
 * Atribui vetor de caminho de colisao do tipo 2 da unidade
 * @param collisionPath Caminho de colisao do tipo 2
 */
void Unit::setCollisionPath(std::vector<std::pair<float,float>> collisionPath) {
    collisionPath_ = collisionPath;
}

/**
 * Atribui tipo do edificio a ser construido pela unidade
 * @param build Tipo do edificio
 */
void Unit::setBuild(int build) {
    build_ = build;
}

/**
 * Atribui ponteiro da mina de ouro a qual a unidade pertence
 * @param goldMine Ponteiro a mina de ouro a qual a unidade pertence
 */
void Unit::setGoldMine(Entity* goldMine) {
    goldMine_ = goldMine;
}

//////////////// Gets sem sets
/**
 * @return Velocidade da unidade
 */
float Unit::getSpeed() {
    return UnitProperty::getInstance()->getSpeed(type_);
}

/**
 * @return Raio da unidade
 */
float Unit::getRadius() {
    return UnitProperty::getInstance()->getRadius(type_);
}

/**
 * @return Alcance da unidade
 */
float Unit::getRange() {
    return UnitProperty::getInstance()->getRange(type_);
}

/**
 * @return Dano da unidade
 */
float Unit::getDamage() {
    return UnitProperty::getInstance()->getDamage(type_);
}

/**
 * @return Velocidade de ataque da unidade
 */
float Unit::getAttackSpeed() {
    return UnitProperty::getInstance()->getAttackSpeed(type_);
}

/**
 * @return Vida maxima da unidade
 */
float Unit::getMaxHp() {
    return UnitProperty::getInstance()->getMaxHp(type_);
}

/**
 * @return Bitmap da unidade
 */
std::vector<ALLEGRO_BITMAP*> Unit::getBitmap() {
    return UnitProperty::getInstance()->getBitmap(type_);
}

/**
 * @return Acoes da unidade
 */
std::vector<std::string> Unit::getAction() {
    return UnitProperty::getInstance()->getAction(type_);
}

/**
 * @return Nome da unidade
 */
std::string Unit::getName() {
    return UnitProperty::getInstance()->getName(type_);
}

/**
 * @return Preco da unidade
 */
int Unit::getCost() {
    return UnitProperty::getInstance()->getCost(type_);
}

/**
 * @return Projetil da unidade
 */
int Unit::getProjectile() {
    return UnitProperty::getInstance()->getProjectile(type_);
}

/**
 * @return Espaco que a unidade ocupa
 */
int Unit::getCount() {
    return UnitProperty::getInstance()->getCount(type_);
}

/**
 * @return Tempo de producao da unidade
 */
float Unit::getTime() {
    return UnitProperty::getInstance()->getTime(type_);
}

/**
 * @return Alcance de agressividade da unidade
 */
float Unit::getAggroRadius() {
    return UnitProperty::getInstance()->getAggroRadius(type_);
}

/**
 * @return true se a unidade esta se movimentando, false caso contrario
 */
bool Unit::isMoving() {
    return (pixelPath_.size() > 0);
}

/**
 * @return true se a unidade esta em colisao do tipo 2, false caso contrario
 */
bool Unit::inCollision() {
    return (collisionPath_.size() > 0);
}

/**
 * @return true se a unidade pode atacar, false caso contrario
 */
bool Unit::isAttackReady() {
    return (cooldown_ == getAttackSpeed());
}

/**
 * Para todas acoes da unidade
 */
void Unit::stop() {
    //path_.clear();
    pixelPath_.clear();
    collisionPath_.clear();
    tx_ = px_;
    ty_ = py_;
    agressive_ = true;
    focus_ = NULL;
    target_ = NULL;
    resetFollow();
}

/**
 * Unidade nao se movimenta por uma atualizacao do jogo
 */
void Unit::wait() {
    wait_ = true;
}

/**
 * Movimenta unidade a uma direcao d
 * @param d Vetor de direcao de movimentacao
 */
void Unit::move(std::pair<float,float> d) {
    px_ = px_ + d.first*getSpeed()/MAX_FPS;
    py_ = py_ + d.second*getSpeed()/MAX_FPS;
    lastMove_ = std::make_pair(d.first*getSpeed()/MAX_FPS, d.second*getSpeed()/MAX_FPS);
}

/**
 * Atualiza movimentacao da unidade
 */
void Unit::updateMove() {
    float tpx, tpy;
    float a,b;
    float error = (getSpeed()/2)/MAX_FPS+1;
    //if (mymath::distance(unit->getPx(),unit->getPy(),unit->getTx(),unit->getTy()) > error) {
    lastMove_.first = 0;
    lastMove_.second = 0;
    if(wait_ == true) {
        wait_ = false;
        return;
    }
    if(pushed_ == true) {
        pushed_ = false;
        deleteFarPath();
    }
    // Colisao do tipo 2
    if(collisionPath_.size() > 0) {
        //printf("colision path\n");
        tpx = collisionPath_[0].first;
        tpy = collisionPath_[0].second;
        a = tpx-px_;
        b = tpy-py_;
        move(std::make_pair(a/sqrt(a*a+b*b), b/sqrt(a*a+b*b)));
        //printf("distance: %f, error = %f\n", mymath::distance(spx,spy,tpx,tpy), error);
        // Verifica se chegou em algum ponto proximo da roda
        if(mymath::distance(px_,py_,tpx,tpy) <= error) {
            if(collisionPath_.size() > 0) {
                collisionPath_.erase(collisionPath_.begin());
            }
        }
        if(collisionPath_.size() == 0) {
            repath_ = true;
        }
    }
    /* Se s anda pro centro de t com uma velocidade speed,
    uma hora s+speed <= speed/2+1 e t sera atingido */
    else if(pixelPath_.size() > 0) {
        std::pair<float,float> nextMove;
        //if(isGroupped()) nextMove = leader_->getNextMovement(); /* Ta bugado ainda, tem q remover o lider qdo muda o path dele e falta remover os paths qdo o lider remove */
        //else nextMove = getNextMovement();
        nextMove = getNextMovement();
        tpx = pixelPath_[0].first;
        tpy = pixelPath_[0].second;
        move(nextMove);
        //printf("distance: %f, error = %f\n", mymath::distance(spx,spy,tpx,tpy), error);
        //printf("%f\n", groupRadius_);
        if(mymath::distance(px_,py_, pixelPath_[pixelPath_.size()-1].first,pixelPath_[pixelPath_.size()-1].second) <= error+groupRadius_) {
            pixelPath_.clear();
            //path_.clear();
        } else if(mymath::distance(px_,py_,tpx,tpy) <= error) {
            if(pixelPath_.size() > 0) {
                //printf("P: %f %f, T: %f %f, error: %f\n", px_, py_, tpx, tpy, error);
                pixelPath_.erase(pixelPath_.begin());
            }
            /*
            if(path_.size() > 0) {
                path_.erase(path_.begin());
            }
            */
        }
        // Depois que acabou a movimentacao a unidade fica no modo agressivo
        if(pixelPath_.size() == 0) {
            agressive_ = true;
        }
    }
}

/**
 * Atualiza tempo de recarga de ataque da unidade
 */
void Unit::updateCooldown() {
    if(cooldown_ < getAttackSpeed()) {
        cooldown_ = cooldown_ + 1.0/(MAX_FPS);
        //printf("CD: %f - div: %f\n", cooldown_, 1.0/(MAX_FPS));
    }
    if(cooldown_ >= getAttackSpeed()) {
        cooldown_ = getAttackSpeed();
    }
}

/**
 * Atualiza angulo e numero de sprite da unidade
 */
void Unit::updateSpriteConfig() {
    float dx = lastMove_.first;
    float dy = lastMove_.second;
    if(renderAct_) {
        actionTime_ = actionTime_ + (5.0/MAX_FPS);
        if(actionTime_ > 2) {
            renderAct_ = false;
            actionTime_ = 0;
        }
    }
    if(target_ != NULL && target_->getHp() > 0) {
        dx = target_->getPx()-px_;
        dy = target_->getPy()-py_;
        movingTime_ = 0;
    } else if(pixelPath_.size() > 0) {
        movingTime_ = (movingTime_ + 5.0/MAX_FPS);
        if(movingTime_ > 4) movingTime_ = 0;
    }
    if(dx != 0 || dy != 0) {
        angle_ = acos(dx/sqrt(dx*dx+dy*dy));
        if(dy < 0) angle_ = -angle_;
    }
    if(pixelPath_.size() == 0) {
        movingTime_ = 0;
    }
}

/**
 * Altera variavel renderAct para true
 */
void Unit::actionSprite() {
    renderAct_ = true;
}

/**
 * Atualiza unidade
 */
void Unit::update() {
    updateMove();
    updateCooldown();
    updateSpriteConfig();
}

/**
 * Devolve proximo movimento da unidade
 * @return Vetor do proximo movimento da unidade
 */
std::pair<float,float> Unit::getNextMovement() {
    float tpx, tpy;
    float a,b;
    if(pixelPath_.size() > 0) {
        tpx = pixelPath_[0].first;
        tpy = pixelPath_[0].second;
        a = tpx-px_;
        b = tpy-py_;
        return std::make_pair(a/sqrt(a*a+b*b), b/sqrt(a*a+b*b));
    }
    return std::make_pair(0,0);
}

/**
 * @return Distancia do caminho a ser percorrido
 */
float Unit::getPixelPathDistance() {
    float soma = 0;
    float currentX = px_;
    float currentY = py_;
    for(int i = 0; i < (int) pixelPath_.size(); i++) {
        soma = soma + mymath::distance(currentX, currentY, pixelPath_[i].first,pixelPath_[i].second);
        currentX = pixelPath_[i].first;
        currentY = pixelPath_[i].second;
    }
    return soma;
}

/**
 * Altera valor de pushed para true
 */
void Unit::checkPath() {
    pushed_ = true;
}

/**
 * Devolve o indice do ponto do vetor de caminho mais proximo a unidade
 * @return Indice descrito
 */
int Unit::getNearestPath() {
    float aux = mymath::distance(px_,py_, pixelPath_[0].first, pixelPath_[0].second);
    for(int i = 1; i < (int) pixelPath_.size(); i++) {
        float dist = mymath::distance(px_,py_, pixelPath_[i].first, pixelPath_[i].second);
        if(aux >= dist) {
            //printf("aux: %f, dist %f\n", aux, dist);
            aux = dist;
        } else {
            return i-1;
        }
    }
    return 0;
}

/**
 * Deleta os caminhos mais longes da unidade
 */
void Unit::deleteFarPath() {
    if(pixelPath_.size() > 1) {
        int pathNum = getNearestPath();
        //printf("%d\n", pathNum);
        if(pathNum > 0) {
            pixelPath_.erase(pixelPath_.begin(), pixelPath_.begin()+pathNum);
        }
    }
}


/**
 * @return true se for unidade, false caso contrario
 */
bool Unit::isUnit() {
    return true;
}

/**
 * Renderiza unidade
 */
void Unit::render() {
    ALLEGRO_COLOR color;
    float percentHP = hp_/getMaxHp();
    if(percentHP >= 0.5) {
        color = al_map_rgb(255*(1-percentHP)*2,255,0);
    } else {
        color = al_map_rgb(255,255*(percentHP-1)*2,0);
    }
    // Sprite
    if(renderAct_) al_draw_rotated_bitmap(getBitmap()[(int) (4+actionTime_)], getRadius(), getRadius(), px_, py_, angle_, 0);
    else al_draw_rotated_bitmap(getBitmap()[(int) movingTime_], getRadius(), getRadius(), px_, py_, angle_, 0);
    // Fundo cinza da barra de HP
    al_draw_line(px_-getRadius(),py_-getRadius(),px_+getRadius(),py_-getRadius(),al_map_rgba(0,0,0,128), 4);
    // Barra de HP
    al_draw_line(px_-getRadius(),py_-getRadius(),px_+getRadius()-(2*getRadius())*(1-percentHP),py_-getRadius(),color, 3);

}

/**
 * @return Coordenada x da entidade que esta sendo seguida
 */
int Unit::getXFollow() {
    return xfollow;
}

/**
 * @return Coordenada y da entidade que esta sendo seguida
 */
int Unit::getYFollow() {
    return yfollow;
}

/**
 * Atribui o valor da coordenada x que esta sendo seguida
 * @param x Coordenada x
 */
void Unit::setXFollow(int x) {
    xfollow = x;
}

/**
 * Atribui o valor da coordenada y que esta sendo seguida
 * @param y Coordenada y
 */
void Unit::setYFollow(int y) {
    yfollow = y;
}

/**
 * Reseta follow
 */
void Unit::resetFollow() {
    follow_ = NULL;
    xfollow = -1;
    yfollow = -1;
}

/**
 * Atribui valor de repath
 * @param repath
 */
void Unit::setRepath(bool repath) {
    repath_ = repath;
}

/**
 * @return true se repath for verdadeiro, false caso contrario
 */
bool Unit::isRepath() {
    return repath_;
}

/**
 * Reseta tratamento de colisao
 */
void Unit::resetCollision() {
    repath_ = false;
    collide = NULL;
    collisionPath_.clear();
    dist = -1;
    ncollide = 0;
}
