#include "BuildingProperty.h"

BuildingProperty* BuildingProperty::instance = NULL;

/**
 * Construtor
 */
BuildingProperty::BuildingProperty() {
    loadCost();
    loadDimension();
    loadTileType();
    loadMaxSlot();
    loadMaxQueue();
    loadMiningGain();
    loadMiningCooldown();
    loadMaxHp();
    loadRadius();
    loadTime();
    loadName();
    loadAction();
    loadBitmap();
    loadIcon();
}

/**
 * Destrutor
 */
BuildingProperty::~BuildingProperty() {
    printf("Deletando bitmaps e instance de BuildingProperty\n");
    for(int i = 0; i < BUILDING_TYPE::b_count; i++) {
        al_destroy_bitmap(bitmap_[i]);
        al_destroy_bitmap(icon_[i]);
    }
}

/**
 * Carrega preco dos edificios
 */
void BuildingProperty::loadCost() {
    cost_[BUILDING_TYPE::HALL] = 5;
    cost_[BUILDING_TYPE::GOLD_MINE] = 3;
    cost_[BUILDING_TYPE::BARRACKS] = 60;
}

/**
 * Carrega dimensao dos edificios
 */
void BuildingProperty::loadDimension() {
    dimension_[BUILDING_TYPE::HALL] = 3;
    dimension_[BUILDING_TYPE::GOLD_MINE] = 3;
    dimension_[BUILDING_TYPE::BARRACKS] = 3;
}

/**
 * Carrega tipo do tile em que o edificio deve ser construido
 */
void BuildingProperty::loadTileType() {
    tileType_[BUILDING_TYPE::HALL] = TileProperty::getInstance()->getTileType(TILE_TYPE::GRASS);
    tileType_[BUILDING_TYPE::GOLD_MINE] = TileProperty::getInstance()->getTileType(TILE_TYPE::GOLD);
    tileType_[BUILDING_TYPE::BARRACKS] = TileProperty::getInstance()->getTileType(TILE_TYPE::GRASS);
}

/**
 * Carrega numero maximo de espaco nos edificios
 */
void BuildingProperty::loadMaxSlot() {
    maxSlot_[BUILDING_TYPE::HALL] = 0;
    maxSlot_[BUILDING_TYPE::GOLD_MINE] = 10;
    maxSlot_[BUILDING_TYPE::BARRACKS] = 0;
}

/**
 * Carrega numero maximo da fila de treinamento dos edificios
 */
void BuildingProperty::loadMaxQueue() {
    maxQueue_[BUILDING_TYPE::HALL] = 6;
    maxQueue_[BUILDING_TYPE::GOLD_MINE] = 0;
    maxQueue_[BUILDING_TYPE::BARRACKS] = 6;
}

/**
 * Carrega quantidade de ouro por intervalo de tempo dos edificios
 */
void BuildingProperty::loadMiningGain() {
    miningGain_[BUILDING_TYPE::HALL] = 0;
    miningGain_[BUILDING_TYPE::GOLD_MINE] = 5;
    miningGain_[BUILDING_TYPE::BARRACKS] = 0;
}

/**
 * Carrega intervalo de tempo de espera para incremento de um tick de ouro
 */
void BuildingProperty::loadMiningCooldown() {
    miningCooldown_[BUILDING_TYPE::HALL] = 0;
    miningCooldown_[BUILDING_TYPE::GOLD_MINE] = 5;
    miningCooldown_[BUILDING_TYPE::BARRACKS] = 0;
}

/**
 * Carrega vida maxima dos edificios
 */
void BuildingProperty::loadMaxHp() {
    maxhp_[BUILDING_TYPE::HALL] = 400;
    maxhp_[BUILDING_TYPE::GOLD_MINE] = 300;
    maxhp_[BUILDING_TYPE::BARRACKS] = 200;
}

/**
 * Carrega raio de colisao dos edificios
 */
void BuildingProperty::loadRadius() {
    radius_[BUILDING_TYPE::HALL] = 24;
    radius_[BUILDING_TYPE::GOLD_MINE] = 24;
    radius_[BUILDING_TYPE::BARRACKS] = 24;
}

/**
 * Carrega tempo de construcao dos edificios
 */
void BuildingProperty::loadTime() {
    time_[BUILDING_TYPE::HALL] = 5;
    time_[BUILDING_TYPE::GOLD_MINE] = 3;
    time_[BUILDING_TYPE::BARRACKS] = 4;
}

/**
 * Carrega nome dos edificios
 */
void BuildingProperty::loadName() {
    name_[BUILDING_TYPE::HALL] = "Hall";
    name_[BUILDING_TYPE::GOLD_MINE] = "Gold_Mine";
    name_[BUILDING_TYPE::BARRACKS] = "Barracks";
}

/**
 * Carrega acoes dos edificios
 */
void BuildingProperty::loadAction() {
    std::vector<std::string> actions;
    actions.push_back("Worker");
    actions.push_back("Rally");
    action_[BUILDING_TYPE::HALL] = actions;
    actions.clear();
    actions.push_back("Unload");
    actions.push_back("Rally");
    action_[BUILDING_TYPE::GOLD_MINE] = actions;
    actions.clear();
    actions.push_back("Warrior");
    actions.push_back("Archer");
    actions.push_back("Rally");
    action_[BUILDING_TYPE::BARRACKS] = actions;
}

/**
 * Carrega bitmap dos edificios
 */
void BuildingProperty::loadBitmap() {
    bitmap_[BUILDING_TYPE::HALL] = al_load_bitmap("pictures/hall.png");
    bitmap_[BUILDING_TYPE::GOLD_MINE] = al_load_bitmap("pictures/gold_mine.png");
    bitmap_[BUILDING_TYPE::BARRACKS] = al_load_bitmap("pictures/barracks.png");
    for(int i = 0; i < BUILDING_TYPE::b_count; i++) {
        if(!bitmap_[i]) {
            printf("Erro ao carregar imagem\n");
        }
    }
}

/**
 * Carrega icones dos edificios
 */
void BuildingProperty::loadIcon() {
    icon_[BUILDING_TYPE::HALL] = al_load_bitmap("pictures/icons/hall.png");
    icon_[BUILDING_TYPE::GOLD_MINE] = al_load_bitmap("pictures/icons/gold_mine.png");
    icon_[BUILDING_TYPE::BARRACKS] = al_load_bitmap("pictures/icons/barracks.png");
    for(int i = 0; i < BUILDING_TYPE::b_count; i++) {
        if(!icon_[i]) {
            printf("Erro ao carregar imagem\n");
        }
    }
}

/**
 * @param type Tipo do edificio
 * @return Preco do edificio do tipo type
 */
int BuildingProperty::getCost(int type) {
    return cost_[type];
}

/**
 * @param type Tipo do edificio
 * @return Dimensao do edificio do tipo type
 */
int BuildingProperty::getDimension(int type) {
    return dimension_[type];
}

/**
 * @param type Tipo do edificio
 * @return Tipo do tile do edificio do tipo type
 */
int BuildingProperty::getTileType(int type) {
    return tileType_[type];
}

/**
 * @param type Tipo do edificio
 * @return Quantidade maxima de espaco do edificio do tipo type
 */
int BuildingProperty::getMaxSlot(int type) {
    return maxSlot_[type];
}

/**
 * @param type Tipo do edificio
 * @return Quantidade maxima da fila de treinamento do edificio do tipo type
 */
int BuildingProperty::getMaxQueue(int type) {
    return maxQueue_[type];
}

/**
 * @param type Tipo do edificio
 * @return Quantidade de ouro ganho por tick do edificio do tipo type
 */
int BuildingProperty::getMiningGain(int type) {
    return miningGain_[type];
}

/**
 * @param type Tipo do edificio
 * @return Tempo de espera para incrementar um tick de ouro do edificio do tipo type
 */
float BuildingProperty::getMiningCooldown(int type) {
    return miningCooldown_[type];
}

/**
 * @param type Tipo do edificio
 * @return Vida maxima do edificio do tipo type
 */
float BuildingProperty::getMaxHp(int type) {
    return maxhp_[type];
}

/**
 * @param type Tipo do edificio
 * @return Tempo de construcao do edificio do tipo type
 */
float BuildingProperty::getTime(int type) {
    return time_[type];
}

/**
 * @param type Tipo do edificio
 * @return Raio do edificio do tipo type
 */
float BuildingProperty::getRadius(int type) {
    return radius_[type];
}

/**
 * @param type Tipo do edificio
 * @return Nome do edificio do tipo type
 */
std::string BuildingProperty::getName(int type) {
    return name_[type];
}

/**
 * @param type Tipo do edificio
 * @return Acoes do edificio do tipo type
 */
std::vector<std::string> BuildingProperty::getAction(int type) {
    return action_[type];
}

/**
 * @param type Tipo do edificio
 * @return Bitmap do edificio do tipo type
 */
ALLEGRO_BITMAP* BuildingProperty::getBitmap(int type) {
    return bitmap_[type];
}

/**
 * @param type Tipo do edificio
 * @return Icone do edificio do tipo type
 */
ALLEGRO_BITMAP* BuildingProperty::getIcon(int type) {
    return icon_[type];
}

/**
 * Devolve o numero do tipo da construcao com nome name
 * @param name Nome da construcao
 * @return Indice do tipo da construcao de nome name, -1 se nao existir
 */
int BuildingProperty::getType(std::string name) {
    for(int i = 0; i < (int) name_.size(); i++) {
        if(name.compare(name_[i]) == 0) return i;
    }
    return -1;
}
