#include "TileProperty.h"

TileProperty* TileProperty::instance = NULL;

/**
 * Construtor
 */
TileProperty::TileProperty() {
    loadBitmap();
    loadWalkable();
    loadChar();
    loadTileType();
}

/**
 * Destrutor
 */
TileProperty::~TileProperty() {
    printf("Deletando bitmaps e instance de TileProperty\n");
    for(int i = 0; i < TILE_TYPE::t_count; i++) {
        al_destroy_bitmap(bitmap_[i]);
    }
    delete instance;
}

/**
 * Carrega tipo dos tiles(utilizado para verificar se e' possivel construir um edificio nesse tipo de tile)
 */
void TileProperty::loadTileType() {
    tileType_[TILE_TYPE::GRASS] = 1;
    tileType_[TILE_TYPE::TREE] = 0;
    tileType_[TILE_TYPE::GOLD] = 2;
}

/**
 * Carrega caractere associado aos tiles
 */
void TileProperty::loadChar() {
    char_[TILE_TYPE::GRASS] = '.';
    char_[TILE_TYPE::TREE] = 't';
    char_[TILE_TYPE::GOLD] = 'g';
}

/**
 * Carrega caracteristica andavel dos tiles
 */
void TileProperty::loadWalkable() {
    walkable_[TILE_TYPE::GRASS] = true;
    walkable_[TILE_TYPE::TREE] = false;
    walkable_[TILE_TYPE::GOLD] = true;
}

/**
 * Carrega bitmap dos tiles
 */
void TileProperty::loadBitmap() {
    bitmap_[TILE_TYPE::GRASS] = al_load_bitmap("pictures/grass.png");
    bitmap_[TILE_TYPE::TREE] = al_load_bitmap("pictures/tree.png");
    bitmap_[TILE_TYPE::GOLD] = al_load_bitmap("pictures/gold.png");
    for(int i = 0; i < TILE_TYPE::t_count; i++) {
        if(!bitmap_[i]) {
            printf("Erro ao carregar imagem\n");

        }
    }
}

/**
 * @param type Tipo do tile
 * @return Tipo do tile(em relacao a construcao)
 */
int TileProperty::getTileType(int type) {
    return tileType_[type];
}

/**
 * @param type Tipo do tile
 * @return Caractere associado ao tile do tipo type
 */
char TileProperty::getChar(int type) {
    return char_[type];
}

/**
 * @param type Tipo do tile
 * @return Caracteristica andavel do tile do tipo type
 */
bool TileProperty::isWalkable(int type) {
    return walkable_[type];
}

/**
 * @param type Tipo do tile
 * @return Bitmap do tile do tipo type
 */
ALLEGRO_BITMAP* TileProperty::getBitmap(int type) {
    return bitmap_[type];
}

/**
 * Devolve o numero do tipo do tile associado ao caractere c
 * @param c Caractere do tile
 * @return Indice do tipo do tile associado ao caractere c, -1 se nao existir
 */
int TileProperty::getType(char c) {
    for(int i = 0; i < TILE_TYPE::t_count; i++) {
        if(char_[i] == c) {
            return i;
        }
    }
    return -1;
}
