#include "UnitProperty.h"

UnitProperty* UnitProperty::instance = NULL;

/**
 * Construtor
 */
UnitProperty::UnitProperty() {
    loadCost();
    loadProjectile();
    loadCount();
    loadMaxHp();
    loadDamage();
    loadAttackSpeed();
    loadRange();
    loadAggroRadius();
    loadSpeed();
    loadRadius();
    loadTime();
    loadName();
    loadAction();
    loadBitmap();
    loadIcon();
}

/**
 * Destrutor
 */
UnitProperty::~UnitProperty() {
    printf("Deletando bitmaps e icons de UnitProperty\n");
    for(int i = 0; i < UNIT_TYPE::u_count; i++) {
        for(int j = 0; j < (int) bitmap_[i].size(); j++) {
            al_destroy_bitmap(bitmap_[i][j]);
        }
        al_destroy_bitmap(icon_[i]);
    }
}

/**
 * Carrega preco das unidades
 */
void UnitProperty::loadCost() {
    cost_[UNIT_TYPE::WORKER] = 5;
    cost_[UNIT_TYPE::WARRIOR] = 20;
    cost_[UNIT_TYPE::ARCHER] = 25;
}

/**
 * Carrega projetil das unidades
 */
void UnitProperty::loadProjectile() {
    projectile_[UNIT_TYPE::WORKER] = PROJECTILE_TYPE::MELEE;
    projectile_[UNIT_TYPE::WARRIOR] = PROJECTILE_TYPE::MELEE;
    projectile_[UNIT_TYPE::ARCHER] = PROJECTILE_TYPE::ARROW;
}

/**
 * Carrega espaco que as unidades ocupam
 */
void UnitProperty::loadCount() {
    count_[UNIT_TYPE::WORKER] = 1;
    count_[UNIT_TYPE::WARRIOR] = 1;
    count_[UNIT_TYPE::ARCHER] = 1;
}

/**
 * Carrega vida maxima das unidades
 */
void UnitProperty::loadMaxHp() {
    maxhp_[UNIT_TYPE::WORKER] = 30;
    maxhp_[UNIT_TYPE::WARRIOR] = 45;
    maxhp_[UNIT_TYPE::ARCHER] = 40;
}

/**
 * Carrega dano de ataque das unidades
 */
void UnitProperty::loadDamage() {
    damage_[UNIT_TYPE::WORKER] = 7;
    damage_[UNIT_TYPE::WARRIOR] = 11;
    damage_[UNIT_TYPE::ARCHER] = 10;
}

/**
 * Carrega tempo de recarga de ataque das unidades
 */
void UnitProperty::loadAttackSpeed() {
    attackspeed_[UNIT_TYPE::WORKER] = 1.5;
    attackspeed_[UNIT_TYPE::WARRIOR] = 1.5;
    attackspeed_[UNIT_TYPE::ARCHER] = 2;
}

/**
 * Carrega alcance das unidades
 */
void UnitProperty::loadRange() {
    range_[UNIT_TYPE::WORKER] = 8+8*2;
    range_[UNIT_TYPE::WARRIOR] = 8+8*2;
    range_[UNIT_TYPE::ARCHER] = 8+16*4;
}

/**
 * Carrega raio de agressividade das unidades
 */
void UnitProperty::loadAggroRadius() {
    aggroradius_[UNIT_TYPE::WORKER] = 8+16*6;
    aggroradius_[UNIT_TYPE::WARRIOR] = 8+16*6;
    aggroradius_[UNIT_TYPE::ARCHER] = 8+16*6;
}

/**
 * Carrega velocidade das unidades
 */
void UnitProperty::loadSpeed() {
    speed_[UNIT_TYPE::WORKER] = 60;
    speed_[UNIT_TYPE::WARRIOR] = 65;
    speed_[UNIT_TYPE::ARCHER] = 60;
}

/**
 * Carrega raio de colisao das unidades
 */
void UnitProperty::loadRadius() {
    radius_[UNIT_TYPE::WORKER] = 8;
    radius_[UNIT_TYPE::WARRIOR] = 8;
    radius_[UNIT_TYPE::ARCHER] = 8;
}

/**
 * Carrega tempo de treinamento das unidades
 */
void UnitProperty::loadTime() {
    time_[UNIT_TYPE::WORKER] = 0.3;
    time_[UNIT_TYPE::WARRIOR] = 3;
    time_[UNIT_TYPE::ARCHER] = 3;
}

/**
 * Carrega nome das unidades
 */
void UnitProperty::loadName() {
    name_[UNIT_TYPE::WORKER] = "Worker";
    name_[UNIT_TYPE::WARRIOR] = "Warrior";
    name_[UNIT_TYPE::ARCHER] = "Archer";
}

/**
 * Carrega acoes das unidades
 */
void UnitProperty::loadAction() {
    std::vector<std::string> actions;
    actions.push_back("Move");
    actions.push_back("Stop");
    actions.push_back("Attack");
    action_[UNIT_TYPE::WARRIOR] = actions;
    action_[UNIT_TYPE::ARCHER] = actions;
    actions.push_back("Gather");
    actions.push_back("Hall");
    actions.push_back("Gold_Mine");
    actions.push_back("Barracks");
    action_[UNIT_TYPE::WORKER] = actions;
}

/**
 * Carrega bitmap das unidades
 */
void UnitProperty::loadBitmap() {
    char buffer[50];
    for(int i = 0; i < 6; i++) {
        sprintf(buffer, "pictures/worker/%d.png",i);
        bitmap_[UNIT_TYPE::WORKER].push_back(al_load_bitmap(buffer));
        sprintf(buffer, "pictures/warrior/%d.png",i);
        bitmap_[UNIT_TYPE::WARRIOR].push_back(al_load_bitmap(buffer));
        sprintf(buffer, "pictures/archer/%d.png",i);
        bitmap_[UNIT_TYPE::ARCHER].push_back(al_load_bitmap(buffer));
    }
    for(int i = 0; i < UNIT_TYPE::u_count; i++) {
        for(int j = 0; j < (int) bitmap_[i].size(); j++) {
            if(!bitmap_[i][j]) {
                printf("Erro ao carregar imagem\n");
                exit(1);
            }
        }
    }
}

/**
 * Carrega iconesdas unidades
 */
void UnitProperty::loadIcon() {
    icon_[UNIT_TYPE::WORKER] = al_load_bitmap("pictures/icons/worker.png");
    icon_[UNIT_TYPE::WARRIOR] = al_load_bitmap("pictures/icons/warrior.png");
    icon_[UNIT_TYPE::ARCHER] = al_load_bitmap("pictures/icons/archer.png");
    for(int i = 0; i < UNIT_TYPE::u_count; i++) {
        if(!icon_[i]) {
            printf("Erro ao carregar imagem\n");
            exit(1);
        }
    }
}

/**
 * @param type Tipo da unidade
 * @return Preco da unidade do tipo type
 */
int UnitProperty::getCost(int type) {
    return cost_[type];
}

/**
 * @param type Tipo da unidade
 * @return Indice do projetil utilizado pela unidade do tipo type
 */
int UnitProperty::getProjectile(int type) {
    return projectile_[type];
}

/**
 * @param type Tipo da unidade
 * @return Espaco que a unidade do tipo type ocupa
 */
int UnitProperty::getCount(int type) {
    return count_[type];
}

/**
 * @param type Tipo da unidade
 * @return Vida maxima da unidade do tipo type
 */
float UnitProperty::getMaxHp(int type) {
    return maxhp_[type];
}

/**
 * @param type Tipo da unidade
 * @return Dano da unidade do tipo type
 */
float UnitProperty::getDamage(int type) {
    return damage_[type];
}

/**
 * @param type Tipo da unidade
 * @return Alcance da unidade do tipo type
 */
float UnitProperty::getRange(int type) {
    return range_[type];
}

/**
 * @param type Tipo da unidade
 * @return Raio de agressividade da unidade do tipo type
 */
float UnitProperty::getAggroRadius(int type) {
    return aggroradius_[type];
}

/**
 * @param type Tipo da unidade
 * @return Velocidade da unidade do tipo type
 */
float UnitProperty::getSpeed(int type) {
    return speed_[type];
}

/**
 * @param type Tipo da unidade
 * @return Velocidade de ataque da unidade do tipo type
 */
float UnitProperty::getAttackSpeed(int type) {
    return attackspeed_[type];
}

/**
 * @param type Tipo da unidade
 * @return Raio de colisao da unidade do tipo type
 */
float UnitProperty::getRadius(int type) {
    return radius_[type];
}

/**
 * @param type Tipo da unidade
 * @return Tempo de treinamento da unidade do tipo type
 */
float UnitProperty::getTime(int type) {
    return time_[type];
}

/**
 * @param type Tipo da unidade
 * @return Nome da unidade do tipo type
 */
std::string UnitProperty::getName(int type) {
    return name_[type];
}

/**
 * @param type Tipo da unidade
 * @return Acoes da unidade do tipo type
 */
std::vector<std::string> UnitProperty::getAction(int type) {
    return action_[type];
}

/**
 * @param type Tipo da unidade
 * @return Bitmap da unidade do tipo type
 */
std::vector<ALLEGRO_BITMAP*> UnitProperty::getBitmap(int type) {
    return bitmap_[type];
}

/**
 * @param type Tipo da unidade
 * @return Icone da unidade do tipo type
 */
ALLEGRO_BITMAP* UnitProperty::getIcon(int type) {
    return icon_[type];
}

/**
 * Devolve o numero do tipo da unidade com nome name
 * @param name Nome da unidade
 * @return Indice do tipo da unidade de nome name, -1 se nao existir
 */
int UnitProperty::getType(std::string name) {
    for(int i = 0; i < (int) name_.size(); i++) {
        if(name.compare(name_[i]) == 0) return i;
    }
    return -1;
}
